/*
 *  BenchManager.java
 *  Created on January 29, 2002, 1:31 PM
 *  Version 1beta0
 *  @author  Marco Parvis
 */

/*
    Questo e' BenchManager una parte di Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
    This is BenchManager part of the Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
 */

/*
    Questo  programma e'  software  libero; e'  lecito redistribuirlo  o
    modificarlo secondo i termini  della Licenza Pubblica Generica GNU
    come e' pubblicata dalla Free  Software Foundation; o la versione 2
    della licenza o (a propria scelta) una versione successiva.
 
    Questo programma  e' distribuito nella  speranza che sia  utile, ma
    SENZA  ALCUNA GARANZIA;  senza  neppure la  garanzia implicita  di
    NEGOZIABILITA'  o di  APPLICABILITA' PER  UN PARTICOLARE  SCOPO.  Si
    veda la Licenza Pubblica Generica GNU per avere maggiori dettagli.
 
    Questo  programma deve  essere  distribuito assieme  ad una  copia
    della Licenza Pubblica Generica GNU;  in caso contrario, se ne puo'
    ottenere  una scrivendo  alla Free  Software Foundation,  Inc., 59
    Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
    L'autore puo' essere contattato a parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 */

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA
 
  The author can be contacted at parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 
 */



package JavaScope;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.swing.*;
import java.security.*;
import java.lang.SecurityManager;

import java.nio.*;

public class BenchManager extends javax.swing.JFrame implements Observer, Runnable {
    private boolean forceSingle=false;
    private boolean forceDouble=false;
    private boolean useProcess=false;
    private String panelH="JavaScope";
    private final int CAL_INPUTS=1;
    private final int CAL_OUTPUTS=2;
    private int calMode=CAL_INPUTS;
    private final boolean calNotImplemented=true;
    private InputManager man;
    private OutputManager outMan;
    
    
    //---------  calibration variables -----------------
    private boolean doNotListen=true;
    private double ch1Cal=0,ch2Cal=0;
    private int nSamples=0;
    
    //    private final int calLength=1000;
    private final double standardInput=0.1;
    private boolean inputsHaveBeenCalibrated=false;
    private boolean removeOffsetInProgress=false;
    
    private double setOutputBits=10000;
    private double[] sampleCh1;//=new double[calLength];
    private double[] sampleCh2;//=new double[calLength];
    
    //---------- calibration values ---------------------
    double calInputCh1=1;
    double calInputCh2=1;
    double calOutputCh1=1;
    double calOutputCh2=1;
    
    private long sCh1=0,sCh2=0;
    private long offsetCalLen=100000;
    
    private String panelIdString="";
    
    private SettingManager sm;
    private Vector panels=new Vector();
    private Vector panelList=new Vector();
    
    private String lastBench;
    private String lastSaveDir="";
    /** Creates new form BenchManager */
    private SplashScreen sp;
    
    private float inputFreq=0;
    
    public BenchManager(String args[]) {
        //decode input commands
        //ByteOrder bo=ByteOrder.nativeOrder();
        //ByteBuffer bb = ByteBuffer.allocateDirect(100);
        //System.out.println("Buffer allocated");
        String s="";
        if (args.length>0)
            for (int i=0;i<args.length;i++){
                if (args[i].startsWith("-s")){
                    
                    forceSingle=true;
                }
                else if (args[i].startsWith("-d")){
                    forceDouble=true;
                }
                else if (args[i].startsWith("-f")){
                    try{
                        String f=(String)args[++i];
                        inputFreq=Float.parseFloat(f);
                        System.out.println("Input frequency forced to: "+inputFreq);
                    }catch(Exception e){
                        System.out.println("-f specified but incorrect frequency given: " + args[i]);
                    }
                } else
                    
                    
                    s=new String((String)args[i]);
            }
        else
            s="";
        //check opearting system
        String actClassPath=System.getProperty("os.name");
        if (System.getProperty("java.version").compareTo("1.4.0")<0){
            System.out.println("This program requires Java version 1.4.0 +");
            System.exit(0);
        }
        if (System.getProperty("os.name").startsWith("Win")){
            System.out.println("It seems we are in an operating system which does not require");
            System.out.println("a separate process for the for output driver");
            if (forceDouble){
                System.out.println("However you have forced double process mode.. ");
                System.out.println("Beware: settings are not saved for the separate process");
                useProcess=true;
            }
            else{
                System.out.println("You may force the use of a double process by using the '-d' switch");
                useProcess=false;
                
            }
        }else{
            System.out.println("It seems we are in an operating system which requires");
            System.out.println("a separate process for the for output driver");
            if (forceSingle){
                System.out.println("However you have forced single process.. You might experience sound problems");
                useProcess=false;
            }else{
                System.out.println("Beware: settings are not saved for the separate process");
                System.out.println("You may force the use of a single process by using the '-s' switch");
                useProcess=true;
            }
        }
        
        String sb;
        sp=new SplashScreen();
        initComponents();
        
        
        
        this.setTitle("Bench Manager");
        this.panelIdString="Bench Manager";
        sm=new SettingManager("MeasSettings2.ini");
        sb=sm.getSettings(this.panelIdString);
        this.applySettings(sb);
        lastBench=s;
        
        SecurityManager security=System.getSecurityManager();
        if (security!=null){
            //we have a security manager installed
            Permission permRec = new javax.sound.sampled.AudioPermission("record");
            Permission permPlay = new javax.sound.sampled.AudioPermission("play");
            try{
                security.checkPermission(permRec);
                security.checkPermission(permPlay);
            }
            catch(Exception e){
                System.out.println("Need record/play permisssions");
                System.exit(-1);
            }
            
            
        }
        
        
        
        
        sp.setMessage("Opening board....");
        
        
        if (inputFreq!=0)
            man= new InputManager(inputFreq);
        else
            man=new InputManager();
        
        outMan=new OutputManager();
        if (!man.lineHasBennOpened()){
            //try loading the debug window
            
            JOptionPane.showMessageDialog(
            this,
            "The input line could not be opened; please have a look at the messages in the console window\n"+
            "To help in the debugging I'll try to load the debug window <Debug.bcm>\n"+
            "if the file is missing or has been moved, try loading the panel yourself\n"+
            "Refer to the manual for hints on solving this problem",
            "Alert",
            JOptionPane.WARNING_MESSAGE);
            loadBench("Debug.bcm");
        }
        else
            if (lastBench.length()>0)
                loadBench(lastBench);
        sp.dispose();
        
        
        this.jRadioButtonCalibrateInputs.setSelected(true);
        this.jRadioButtonCalibrateOutputs.setEnabled(false);
        calMode=CAL_INPUTS;
        this.jProgressBarEstim.setMinimum(0);
        this.jProgressBarEstim.setMaximum(100);
        this.jProgressBarEstim.setVisible(false);
        this.jLabelStatus.setVisible(true);
        this.jLabelStatus.setText(" ");
        this.jLabelMessage.setVisible(false);
        this.jLabelOffset1.setText("Ch1 offset: NA");
        this.jLabelOffset2.setText("Ch2 offset: NA");
        if (calNotImplemented)
            this.jButtonDoCalibration.setEnabled(false);
        
        this.show();
        
    }
    private void loadBench(String filename){
        //now load the panels
        
        //System.out.println("Loading: "+filename);
        if (!openBenchFile(filename)){
            JOptionPane.showMessageDialog(
            this,
            "Could not find file: "+filename+"\n Bench loading aborted",
            "Alert",
            JOptionPane.WARNING_MESSAGE);
            
            return;
        }
        if (panels.size()>0){
            boolean saveOpt=false;
            String sb;
            outMan.stopPlay();
            
            int n = JOptionPane.showConfirmDialog(
            this,
            "Do you want to save the settings for actual panels ?",
            "Alert",
            JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
            if (n==JOptionPane.YES_OPTION)
                saveOpt=true;
            for (int i=0;i<panels.size();i++){
                String ss=panels.get(i).getClass().getName();
                if (ss.startsWith(panelH)){
                    VirtualPanel vp=(VirtualPanel)panels.get(i);
                    sb=vp.getSettings();
                    if (saveOpt) sm.updateSettings(sb);
                    ((JFrame)vp).dispose();
                }
            }
            
            panels.removeAllElements();
        }
        
        
        String sb;
        Object vp;
        for (int i=0;i<panelList.size();i++){
            StringTokenizer ss=new StringTokenizer((String)panelList.get(i),",");
            String panelClass=ss.nextToken();
            String panelName=ss.nextToken();
            if (sp!=null) sp.setMessage("Loading "+ panelName+" ....");
            vp= activatePanel(panelClass, panelName);
            if (vp!=null){
                panels.add(vp);
            }
        }
        for (int i=0;i<panels.size();i++){
            String ss=panels.get(i).getClass().getName();
            
            if (ss.startsWith(panelH))
                ((JFrame)panels.get(i)).show();
            
        }
    }
    private Object activatePanel(String p,String panelName){
        Class[] cl;
        Method[] ml;
        Object pl=null;
        String sb;
        String specialClass="JavaScope.JFrameFunctionGenerator";
        if (useProcess &  p.compareTo(specialClass)==0 ){
            String actClassPath=System.getProperty("java.class.path");
            String javaHome=System.getProperty("java.home");
            
            StringBuffer tmpBuffer=new StringBuffer(javaHome);
            tmpBuffer.append("/bin/java -cp ");
            tmpBuffer.append(actClassPath);
            tmpBuffer.append(" ");
            tmpBuffer.append(specialClass);
            try {
                System.out.println("Starting "+tmpBuffer);
                Runtime r=Runtime.getRuntime();
                Process child=r.exec(tmpBuffer.toString());
                
                
                return child;
            }
            catch(java.io.IOException e) {
                System.err.println(
                "IOException starting process: "+e.toString());
                return null;
            }
            
        }
        
        
        try{
            //check for implemented interface
            cl=Class.forName(p).getInterfaces();
            for (int i=0; i<cl.length;i++){
                String nm=cl[i].getName();
                if (nm.compareTo("JavaScope.VirtualPanel")==0){
                    VirtualPanel vpi=(VirtualPanel)Class.forName(p).newInstance();
                    
                    vpi.setInputManager(man);
                    vpi.setOutputManager(outMan);
                    vpi.setIdString(panelName);
                    sb=sm.getSettings(panelName);
                    vpi.applySettings(sb);
                    vpi.setBenchComponent(true);
                    return vpi;
                }
            }
        }
        catch(Throwable t){
            int n = JOptionPane.showConfirmDialog(
            this,
            "Panel "+panelName +"( class: "+p+" )\n Could not be loaded due to the following exception\n"+
            t.toString()+"\n Do you want to continue ?",
            "Alert",
            JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
            if (n==JOptionPane.NO_OPTION)
                System.exit(0);
        }
        return null;
        
    }
    
    private boolean openBenchFile(String filename) {
        String tmpStr;
        BufferedReader in;
        try {
            in
            = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
        } catch (FileNotFoundException fe){
            //file does notexist
            panelList.removeAllElements();
            return false;
        } catch (Throwable ie){ //catch (IOException ie){
            System.out.println(ie.toString());
            return false;
        }
        try {
            panelList.removeAllElements();
            while ((tmpStr=in.readLine())!=null){
                if (tmpStr.length()>0){
                    panelList.add(tmpStr);
                }
            }
            in.close();
            return true;
            
        }
        catch (Throwable ie){ //catch (IOException ie){
            System.out.println(ie.toString());}
        return false;
    }
    
    
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    
    
    public String getSettings(){
        int x=this.getLocation().x;
        int y=this.getLocation().y;
        int w=this.getSize().width;
        int h=this.getSize().height;
        
        String s=panelIdString+"|Position="+String.valueOf(x)+","+String.valueOf(y)+"|Size="+String.valueOf(w)+","+String.valueOf(h)+"|"+
        "LastDir="+lastSaveDir;
        return s;
    }
    public void applySettings(String settings){
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        StringTokenizer tokens=new StringTokenizer(settings,"|");
        while (tokens.hasMoreTokens()){
            String token=tokens.nextToken();
            //browse through recognized tokens
            if (token.indexOf("Position=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<1) x=1;
                    if (x>screenSize.width-10) x=screenSize.width-10;
                    if (y<1) y=1;
                    if (y>screenSize.height-10) y=screenSize.height-10;
                    this.setLocation(x,y);
                }catch (Throwable t){}
                
            }else if (token.indexOf("Size=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<10) x=10;
                    if (x>screenSize.width-20) x=screenSize.width-20;
                    if (y<10) y=10;
                    if (y>screenSize.height-20) y=screenSize.height-20;
                    this.setSize(x,y);
                }catch (Throwable t){}
            }else if (token.indexOf("LastDir=")>=0){
                String[] p=getValues(token);
                try{
                    this.lastSaveDir=p[0];
                }catch (Throwable t){}
            }
            
        }
    }
    private String[] getValues(String s){
        int peq=s.indexOf("=");
        if (peq>0){
            StringTokenizer ss=new StringTokenizer(s.substring(peq+1),",");
            if (ss.countTokens()>0){
                String[] vl=new String[ss.countTokens()];
                for (int i=0;i<vl.length;i++)
                    vl[i]=ss.nextToken();
                return vl;
            }else
                return new String[0];
            
        }else
            return new String[0];
    }
    
    private void saveThisTrace(){
        ///
        JFileChooser fc=new JFileChooser();
        fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File f) {
                if (f.isDirectory()) {
                    return true;
                }
                String name = f.getName();
                if (name.endsWith(".txt")) {
                    return true;
                }
                return false;
            }
            public String getDescription() {
                return "Sample data files  (*.txt) ";
            }
        });
        fc.setCurrentDirectory(new File(lastSaveDir));
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
            String filename=new String(fc.getSelectedFile().getAbsolutePath());
            try {
                PrintWriter out
                = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
                for (int i=0;i<sampleCh1.length;i++){
                    out.print(sampleCh1[i]);
                    out.print("   ");
                    out.println(sampleCh2[i]);
                }
                
                out.close();}
            catch (IOException ie){
                System.out.println(ie.toString());
            }
            
            
        }
    }
    
    
    
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupCalType = new javax.swing.ButtonGroup();
        jPanelCalibration = new javax.swing.JPanel();
        jRadioButtonCalibrateInputs = new javax.swing.JRadioButton();
        jRadioButtonCalibrateOutputs = new javax.swing.JRadioButton();
        jLabelMessage = new javax.swing.JLabel();
        jButtonDoCalibration = new javax.swing.JButton();
        jButtonInputOffset = new javax.swing.JButton();
        jCheckBoxRemoveOffset = new javax.swing.JCheckBox();
        jLabelOffset1 = new javax.swing.JLabel();
        jLabelOffset2 = new javax.swing.JLabel();
        jProgressBarEstim = new javax.swing.JProgressBar();
        jLabelStatus = new javax.swing.JLabel();
        jToolBarBench = new javax.swing.JToolBar();
        jButtonLoad = new javax.swing.JButton();
        jButtonGpl = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jPanelCalibration.setLayout(new java.awt.GridBagLayout());

        jPanelCalibration.setBorder(new javax.swing.border.TitledBorder("Calibration Manager"));
        jRadioButtonCalibrateInputs.setText("Calibrate Inputs");
        buttonGroupCalType.add(jRadioButtonCalibrateInputs);
        jRadioButtonCalibrateInputs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonCalibrateInputsActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanelCalibration.add(jRadioButtonCalibrateInputs, gridBagConstraints);

        jRadioButtonCalibrateOutputs.setText("Calibrate Outputs");
        buttonGroupCalType.add(jRadioButtonCalibrateOutputs);
        jRadioButtonCalibrateOutputs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonCalibrateOutputsActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanelCalibration.add(jRadioButtonCalibrateOutputs, gridBagConstraints);

        jLabelMessage.setText("Connects inputs to standard");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(20, 0, 0, 0);
        jPanelCalibration.add(jLabelMessage, gridBagConstraints);

        jButtonDoCalibration.setText("Do Calibration");
        jButtonDoCalibration.setPreferredSize(new java.awt.Dimension(90, 26));
        jButtonDoCalibration.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDoCalibrationActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 0, 5);
        jPanelCalibration.add(jButtonDoCalibration, gridBagConstraints);

        jButtonInputOffset.setText("Estimate Offset");
        jButtonInputOffset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonInputOffsetActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        jPanelCalibration.add(jButtonInputOffset, gridBagConstraints);

        jCheckBoxRemoveOffset.setText("Remove Offset");
        jCheckBoxRemoveOffset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxRemoveOffsetActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        jPanelCalibration.add(jCheckBoxRemoveOffset, gridBagConstraints);

        jLabelOffset1.setText("00000");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 20;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelCalibration.add(jLabelOffset1, gridBagConstraints);

        jLabelOffset2.setText("00000");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipadx = 20;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanelCalibration.add(jLabelOffset2, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 5);
        jPanelCalibration.add(jProgressBarEstim, gridBagConstraints);

        jLabelStatus.setLabelFor(jProgressBarEstim);
        jLabelStatus.setText("Status");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        jPanelCalibration.add(jLabelStatus, gridBagConstraints);

        getContentPane().add(jPanelCalibration, java.awt.BorderLayout.CENTER);

        jButtonLoad.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/open.gif")));
        jButtonLoad.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonLoad.setMinimumSize(new java.awt.Dimension(24, 20));
        jButtonLoad.setPreferredSize(new java.awt.Dimension(20, 24));
        jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonLoadActionPerformed(evt);
            }
        });

        jToolBarBench.add(jButtonLoad);

        jButtonGpl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/gpl.gif")));
        jButtonGpl.setToolTipText("show licence");
        jButtonGpl.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonGpl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGplActionPerformed(evt);
            }
        });

        jToolBarBench.add(jButtonGpl);

        getContentPane().add(jToolBarBench, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents
    
    private void jCheckBoxRemoveOffsetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxRemoveOffsetActionPerformed
        if (this.jCheckBoxRemoveOffset.isSelected())
            man.useInputOffset(true);
        else
            man.useInputOffset(false);
        // Add your handling code here:
    }//GEN-LAST:event_jCheckBoxRemoveOffsetActionPerformed
    
    private void jButtonInputOffsetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonInputOffsetActionPerformed
        this.jButtonInputOffset.setEnabled(false);
        this.jButtonDoCalibration.setEnabled(false);
        this.jProgressBarEstim.setVisible(true);
        this.jLabelStatus.setText("Working...");
        this.jLabelStatus.setVisible(true);
        startRemoveOffset();
        // Add your handling code here:
    }//GEN-LAST:event_jButtonInputOffsetActionPerformed
    
    private void jButtonGplActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGplActionPerformed
        // Add your handling code here:
        new GPL();
    }//GEN-LAST:event_jButtonGplActionPerformed
    
    private void jButtonLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonLoadActionPerformed
        JFileChooser fc=new JFileChooser();
        fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File f) {
                if (f.isDirectory()) {
                    return true;
                }
                String name = f.getName();
                if (name.endsWith(".bcm")) {
                    return true;
                }
                return false;
            }
            public String getDescription() {
                return "Bench description files  (*.bcm) ";
            }
        });
        fc.setCurrentDirectory(new File(lastSaveDir));
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
            System.out.println("Loading bench "+fc.getSelectedFile().getAbsolutePath());
            loadBench(fc.getSelectedFile().getAbsolutePath());
            this.lastSaveDir=new String(fc.getCurrentDirectory().toString());
            //System.out.println(this.lastSaveDir);
        }
    }//GEN-LAST:event_jButtonLoadActionPerformed
    
    private void jButtonDoCalibrationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDoCalibrationActionPerformed
        switch (calMode){
            case CAL_INPUTS:
                startInputCalibration();
                break;
            case CAL_OUTPUTS:
                startOutputCalibration();
                break;
            default:
                break;
        }
        
    }//GEN-LAST:event_jButtonDoCalibrationActionPerformed
    
    private void jRadioButtonCalibrateOutputsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonCalibrateOutputsActionPerformed
        if (jRadioButtonCalibrateOutputs.isSelected()){
            calMode=CAL_OUTPUTS;
            jLabelMessage.setText("Connect inputs to outputs");
        }
        
        
    }//GEN-LAST:event_jRadioButtonCalibrateOutputsActionPerformed
    
    private void jRadioButtonCalibrateInputsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonCalibrateInputsActionPerformed
        if (jRadioButtonCalibrateInputs.isSelected()){
            calMode=CAL_INPUTS;
            jLabelMessage.setText("Connect inputs to standard");
        }
    }//GEN-LAST:event_jRadioButtonCalibrateInputsActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        String sb;
        SettingManager sm=new SettingManager("MeasSettings2.ini");
        //save settings for all panels
        for (int i=0;i<panels.size();i++){
            String ss=panels.get(i).getClass().getName();
            
            if (ss.startsWith(panelH)){
                VirtualPanel vp=(VirtualPanel)panels.get(i);
                sb=vp.getSettings();
                sm.updateSettings(sb);
                ((JFrame)vp).dispose();
            } else if (ss.indexOf("Process")!=0){
                Process p=(Process)panels.get(i);
                p.destroy();
            }
            
        }
        //save our settings
        sb=this.getSettings();
        sm.updateSettings(sb);
        
        sm.saveSettings();
        man.stopCapture();
        outMan.stopPlay();
        
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        if (args.length>0)
            for (int i=0;i<args.length;i++){
                if (args[i].startsWith("-h")){
                    System.out.println("Usage: /JAVAHOME/java -jar JavaMeas.jar [options] benchFile");
                    System.out.println("options: \t -s forces single process mode (def for Win environment)");
                    System.out.println("options: \t -d forces double process mode (def for non Win environment)");
                    System.out.println("\t -h print this help");
                    System.exit(0);
                }
            }
        
        new BenchManager(args).show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupCalType;
    private javax.swing.JButton jButtonDoCalibration;
    private javax.swing.JButton jButtonGpl;
    private javax.swing.JButton jButtonInputOffset;
    private javax.swing.JButton jButtonLoad;
    private javax.swing.JCheckBox jCheckBoxRemoveOffset;
    private javax.swing.JLabel jLabelMessage;
    private javax.swing.JLabel jLabelOffset1;
    private javax.swing.JLabel jLabelOffset2;
    private javax.swing.JLabel jLabelStatus;
    private javax.swing.JPanel jPanelCalibration;
    private javax.swing.JProgressBar jProgressBarEstim;
    private javax.swing.JRadioButton jRadioButtonCalibrateInputs;
    private javax.swing.JRadioButton jRadioButtonCalibrateOutputs;
    private javax.swing.JToolBar jToolBarBench;
    // End of variables declaration//GEN-END:variables
    
    public void update(Observable o, Object arg){
        //new data arrived arg is  short[] and contains the samples in sequence
        if (doNotListen)
            return;
        short[] dataS=(short[])arg;
        if (removeOffsetInProgress){
            for (int i=0;i<dataS.length/2;i++){
                sCh1+=(long)dataS[2*i];
                sCh2+=(long)dataS[2*i+1];
            }
            nSamples+=dataS.length/2;
            if (nSamples>=this.offsetCalLen){
                processInputOffset();
                man.deleteObserver(this);
                doNotListen=true;
                this.jButtonInputOffset.setEnabled(true);
                if (!calNotImplemented)
                    this.jButtonDoCalibration.setEnabled(true);
                this.removeOffsetInProgress=false;
                this.jProgressBarEstim.setValue(0);
                this.jProgressBarEstim.setVisible(false);
                this.jLabelStatus.setText(" ");
                
            }
            else{
                this.jProgressBarEstim.setValue((int)(100*nSamples/offsetCalLen));
            }
        }
        else{
            
            sampleCh1=new double[dataS.length/2];
            sampleCh2=new double[dataS.length/2];
            //compute the rms value
            for (int i=0;i<dataS.length/2;i++){
                sampleCh1[i]=(double)dataS[2*i];
                sampleCh2[i]=(double)dataS[2*i+1];
            }
            saveThisTrace();
            //compute the rms value
            for (int i=0;i<dataS.length/2;i+=2){
                ch1Cal+=((double)dataS[i])*dataS[i];
                ch2Cal+=((double)dataS[i+1])*dataS[i+1];
            }
            man.deleteObserver(this);
            doNotListen=true;
        }
        
    }
    private void startRemoveOffset(){
        nSamples=0;
        sCh1=0;
        sCh2=0;
        outMan.stopPlay();
        man.useInputOffset(false);
        removeOffsetInProgress=true;
        doNotListen=false;
        man.addObserver(this);
        
    }
    private void startInputCalibration(){
        ch1Cal=ch2Cal=nSamples=0;
        //calLength=44100;
        doNotListen=false;
        man.addObserver(this);
        
    }
    private void stopInputCalibration(){
        man.deleteObserver(this);
    }
    private void startOutputCalibration(){
        man.addObserver(this);
    }
    private void stopOutputCalibration(){
        man.deleteObserver(this);
    }
    private void processInputOffset(){
        short mCh1=(short)(sCh1/nSamples);
        short mCh2=(short)(sCh2/nSamples);
        this.jLabelOffset1.setText("Ch1 offset:"+Short.toString(mCh1));
        this.jLabelOffset2.setText("Ch2 offset:"+Short.toString(mCh2));
        
        //System.out.println(mCh1+"  "+mCh2);
        man.setInputOffset(mCh1,mCh2);
        if (this.jCheckBoxRemoveOffset.isSelected())
            man.useInputOffset(true);
        
        
    }
    private void processCalData(){
        switch (calMode){
            case CAL_INPUTS: {
                int i;
                //sampleCh1 and sampleCh2 contain the samples
                //get max and min
                double chMax=sampleCh1[0];
                double chMin=sampleCh1[0];
                
                for (i=1;i<sampleCh1.length;i++){
                    if (sampleCh1[i]>chMax)
                        chMax=sampleCh1[i];
                    if (sampleCh1[i]<chMin)
                        chMin=sampleCh1[i];
                }
                
                
                //we have the rms value of measured inputs
                //and the input voltage
                calInputCh1=standardInput/ch1Cal; //expressed as volt/bit
                calInputCh2=standardInput/ch2Cal;
                inputsHaveBeenCalibrated=true;
                this.jRadioButtonCalibrateOutputs.setEnabled(true);
            }
            break;
            case CAL_OUTPUTS:
                //we must have the input already calibrated
                //to be here
                double actInputCh1=ch1Cal*calInputCh1;
                double actInputCh2=ch2Cal*calInputCh2;
                
                calOutputCh1=actInputCh1/setOutputBits;
                calOutputCh2=actInputCh1/setOutputBits;
                
                break;
            default:
                break;
        }
    }
    
    
    private boolean openFile(String filename) {
        String tmpStr;
        BufferedReader in;
        try {
            in
            = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
        } catch (FileNotFoundException fe){
            //file does not exist
            
            return false;
        } catch (Throwable ie){ //catch (IOException ie){
            System.out.println(ie.toString());
            return false;
        }
        try {
            tmpStr=in.readLine();
            in.close();
            return true;
        }
        catch (Throwable ie){ //catch (IOException ie){
            System.out.println(ie.toString());}
        return false;
    }
    private void saveFile(String filename)  {
        try {
            PrintWriter out
            = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
            out.print("   ");
            
            out.close();}
        catch (IOException ie){
            System.out.println(ie.toString());
        }
    }
    
    public void run() {
    }
    
    
    
    
}
