/*
 *  JFrameDmm.java
 *  Created on January 29, 2002, 1:31 PM
 *  Version 1beta0
 *  @author  Marco Parvis
 */

/*
    Questo e' JFrameDmm una parte di Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
    This is JFrameDmm part of the Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
 */

/*
    Questo  programma e'  software  libero; e'  lecito redistribuirlo  o
    modificarlo secondo i termini  della Licenza Pubblica Generica GNU
    come e' pubblicata dalla Free  Software Foundation; o la versione 2
    della licenza o (a propria scelta) una versione successiva.
 
    Questo programma  e' distribuito nella  speranza che sia  utile, ma
    SENZA  ALCUNA GARANZIA;  senza  neppure la  garanzia implicita  di
    NEGOZIABILITA'  o di  APPLICABILITA' PER  UN PARTICOLARE  SCOPO.  Si
    veda la Licenza Pubblica Generica GNU per avere maggiori dettagli.
 
    Questo  programma deve  essere  distribuito assieme  ad una  copia
    della Licenza Pubblica Generica GNU;  in caso contrario, se ne puo'
    ottenere  una scrivendo  alla Free  Software Foundation,  Inc., 59
    Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
    L'autore puo' essere contattato a parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 */

/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
 
    The author can be contacted at parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 
 */



/*
 * This panel implements a Digital Multimeter for AC Valtages
 * It is based on two cascaded blocks
 * The first block is an AC/DC converter that can work with different approaches
 * The second block implements an AD Converter that resembles a double slope ADC
 * The implemeted conversions are:
 *      RMS
 *      AVG (since signals does not have DC components it does not matter full wave or half wave
 *      PEAK detector with series diode (positive and negative)
 *      PEAK detector with parallel diode (positive and negative)
 * The ADC integration time is expressed in PLC (period line count) based on 50Hz mains
 */

package JavaScope;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.text.*;




public class JFrameDmm extends javax.swing.JFrame implements VirtualPanel{ //implements Observer{
    private boolean weAreInABench=false;
    private String panelIdString;
    
    private boolean flashOn=false;
    InputManager man;
    private boolean useCalibrationFactor=false;
    private double calFactorVoltAtBitCh1=1;
    private double calFactorVoltAtBitCh2=1;
    
    private final int AVG_MODE=1;
    private final int RMS_MODE=2;
    private final int PEAK_MODE_1=3; //series plus
    private final int PEAK_MODE_2=4; //series minus
    private final int PEAK_MODE_3=5; //parallel plus
    private final int PEAK_MODE_4=6; //parallel minus
    private final int DC_MODE=7;  //act as a DC dmm
    private int actualMode=RMS_MODE;
    private final double AVG2RMS=1.1107207;
    private final double PEAK2RMS=0.7071068;
    
    //peak computing values
    private int peakPos=0;
    private int peakVal=0;
    private int peakNegativeVal=0;
    private double tauPeak=1.0/44100.0; //1s
    private double correctionFactor=1.0005; //exp(-0.001)/2 i.e. 1s at 1 kHz
    private final int PLC_SPOT=0;
    private final int PLC_001=1;
    private final int PLC_01=2;
    private final int PLC_1=3;
    private final int PLC_2=4;
    private final int PLC_5=5;
    private final int PLC_10=6;
    private final int PLC_100=7;
    private final int PLC_MANUAL=8;
    
    private final int samplingFreq=44100;
    private final int lineFreq=50;
    
    private boolean measStarted=false;
    private long saveIntermediate;
    private int gotSamples;
    private int actualChannel=0;
    private int actualNumberOfSamples=0;
    
    private java.util.Timer timerMonitor;
    private DmmMonitor dmmMonitor;
    private short[] dataS;
    
    
    
    /** Creates new form JFrameDmm */
    public String getSettings(){
        
        int x=this.getLocation().x;
        int y=this.getLocation().y;
        int w=this.getSize().width;
        int h=this.getSize().height;
        
        String s=panelIdString+"|Position="+String.valueOf(x)+","+String.valueOf(y)+"|Size="+String.valueOf(w)+","+String.valueOf(h)+"|";
        if (this.useCalibrationFactor)
            s=s+"UseInputCal=1|";
        else
            s=s+"UseInputCal=0|";
        s=s+"InputCalFactors="+String.valueOf(this.calFactorVoltAtBitCh1)+","+String.valueOf(this.calFactorVoltAtBitCh2);
        return s;
    }
    public void applySettings(String settings){
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        
        StringTokenizer tokens=new StringTokenizer(settings,"|");
        while (tokens.hasMoreTokens()){
            String token=tokens.nextToken();
            //browse through recognized tokens
            if (token.indexOf("Position=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<1) x=1;
                    if (x>screenSize.width-10) x=screenSize.width-10;
                    if (y<1) y=1;
                    if (y>screenSize.height-10) y=screenSize.height-10;
                    
                    this.setLocation(x,y);
                }catch (Throwable t){}
                
            }else if (token.indexOf("Size=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<10) x=10;
                    if (x>screenSize.width-20) x=screenSize.width-20;
                    if (y<10) y=10;
                    if (y>screenSize.height-20) y=screenSize.height-20;
                    this.setSize(x,y);
                }catch (Throwable t){}
            }else if (token.indexOf("UseInputCal=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    if (x!=0)
                        this.useCalibrationFactor=true;
                    else
                        this.useCalibrationFactor=false;
                }catch (Throwable t){}
            }else if (token.indexOf("InputCalFactors=")>=0){
                String[] p=getValues(token);
                try{
                    double x=new Double(p[0]).doubleValue();
                    double y=new  Double(p[1]).doubleValue();
                    this.calFactorVoltAtBitCh1=x;
                    this.calFactorVoltAtBitCh2=y;
                }catch (Throwable t){}
            }
        }
    }
    private String[] getValues(String s){
        int peq=s.indexOf("=");
        if (peq>0){
            StringTokenizer ss=new StringTokenizer(s.substring(peq+1),",");
            if (ss.countTokens()>0){
                String[] vl=new String[ss.countTokens()];
                for (int i=0;i<vl.length;i++)
                    vl[i]=ss.nextToken();
                return vl;
            }else
                return new String[0];
            
        }else
            return new String[0];
    }
    public void useOutputCalibration(boolean use){
    }
    public void useInputCalibration(boolean use){
        this.useCalibrationFactor=use;
    }
    public void setOutputCalibration(double newCalCh1,double newCalCh2){
    }
    public void setInputCalibration(double newCalCh1,double newCalCh2){
        this.calFactorVoltAtBitCh1=newCalCh1;
        this.calFactorVoltAtBitCh2=newCalCh2;
    }
    
    
    public void setOutputManager(OutputManager im){
    }
    public void setInputManager(InputManager im){
        this.man=im;
        man.addObserver(this);
    }
    
    public void setBenchComponent(boolean bc){
        weAreInABench=bc;
        if (bc)
            this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        else
            this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
    }
    public void setIdString(String s){
        this.panelIdString=s;
        this.setTitle(s);
    }
    public String getIdString(){
        return this.panelIdString;
    }
    
    
    public JFrameDmm(InputManager im) {
        this.man=im;
        initComponents();
        man.addObserver(this);
        this.jComboBoxPLC.addItem("Spot");
        this.jComboBoxPLC.addItem("0.01");
        this.jComboBoxPLC.addItem("0.1");
        this.jComboBoxPLC.addItem("1");
        this.jComboBoxPLC.addItem("2");
        this.jComboBoxPLC.addItem("5");
        this.jComboBoxPLC.addItem("10");
        this.jComboBoxPLC.addItem("100");
        
        this.jComboBoxPLC.addItem("Manual");
        this.jComboBoxPLC.setSelectedIndex(PLC_1);
        this.jTextFieldIntTime.setEnabled(false);
        this.jButtonSetIntTime.setEnabled(false);
        this.jToggleButtonRMS.setSelected(true);
        actualMode=RMS_MODE;
        this.jToggleButtonCh1.setSelected(true);
        actualNumberOfSamples=samplingFreq/lineFreq; //one plc
        weAreInABench=false;
        dmmMonitor=new DmmMonitor();
        timerMonitor=new java.util.Timer();
        timerMonitor.scheduleAtFixedRate(dmmMonitor,250,250);
    }
    public JFrameDmm() {
        initComponents();
        this.jComboBoxPLC.addItem("Spot");
        this.jComboBoxPLC.addItem("0.01");
        this.jComboBoxPLC.addItem("0.1 ");
        this.jComboBoxPLC.addItem("1   ");
        this.jComboBoxPLC.addItem("2   ");
        this.jComboBoxPLC.addItem("5   ");
        this.jComboBoxPLC.addItem("10  ");
        this.jComboBoxPLC.addItem("100 ");
        
        this.jComboBoxPLC.addItem("Manual");
        this.jComboBoxPLC.setSelectedIndex(PLC_1);
        this.jTextFieldIntTime.setEnabled(false);
        this.jButtonSetIntTime.setEnabled(false);
        this.jToggleButtonRMS.setSelected(true);
        actualMode=RMS_MODE;
        this.jToggleButtonCh1.setSelected(true);
        actualNumberOfSamples=samplingFreq/lineFreq; //one plc
        dmmMonitor=new DmmMonitor();
        timerMonitor=new java.util.Timer();
        timerMonitor.scheduleAtFixedRate(dmmMonitor,250,250);
        ((SignalPanel)this.jPanelLamp).setCentered(true);
        weAreInABench=false;
        
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupAcMode = new javax.swing.ButtonGroup();
        buttonGroupChannels = new javax.swing.ButtonGroup();
        jTextFieldResult = new javax.swing.JTextField();
        jComboBoxPLC = new javax.swing.JComboBox();
        jToggleButtonAVG = new javax.swing.JToggleButton();
        jToggleButtonRMS = new javax.swing.JToggleButton();
        jTextFieldIntTime = new javax.swing.JTextField();
        jButtonSetIntTime = new javax.swing.JButton();
        jToggleButtonCh1 = new javax.swing.JToggleButton();
        jToggleButtonCh2 = new javax.swing.JToggleButton();
        jPanelLamp = new SignalPanel();
        jToggleButtonPeak1 = new javax.swing.JToggleButton();
        jToggleButtonPeak2 = new javax.swing.JToggleButton();
        jToggleButtonPeak3 = new javax.swing.JToggleButton();
        jToggleButton3 = new javax.swing.JToggleButton();
        jToggleButtonDC = new javax.swing.JToggleButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jTextFieldResult.setBackground(java.awt.Color.black);
        jTextFieldResult.setEditable(false);
        jTextFieldResult.setFont(new java.awt.Font("Dialog", 1, 18));
        jTextFieldResult.setForeground(java.awt.Color.green);
        jTextFieldResult.setText("000000");
        jTextFieldResult.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        jTextFieldResult.setMargin(new java.awt.Insets(0, 0, 5, 0));
        jTextFieldResult.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldResultActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 2;
        gridBagConstraints.ipady = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 0);
        getContentPane().add(jTextFieldResult, gridBagConstraints);

        jComboBoxPLC.setToolTipText("Integration period");
        jComboBoxPLC.setMinimumSize(new java.awt.Dimension(31, 20));
        jComboBoxPLC.setPreferredSize(new java.awt.Dimension(70, 20));
        jComboBoxPLC.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxPLCActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(jComboBoxPLC, gridBagConstraints);

        jToggleButtonAVG.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/abs.gif")));
        jToggleButtonAVG.setToolTipText("Average");
        buttonGroupAcMode.add(jToggleButtonAVG);
        jToggleButtonAVG.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonAVG.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonAVG.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonAVG.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonAVGActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        getContentPane().add(jToggleButtonAVG, gridBagConstraints);

        jToggleButtonRMS.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/rms.gif")));
        jToggleButtonRMS.setToolTipText("RMS");
        buttonGroupAcMode.add(jToggleButtonRMS);
        jToggleButtonRMS.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonRMS.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonRMS.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonRMS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonRMSActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        getContentPane().add(jToggleButtonRMS, gridBagConstraints);

        jTextFieldIntTime.setText("10");
        jTextFieldIntTime.setToolTipText("Int interval (ms)");
        jTextFieldIntTime.setPreferredSize(new java.awt.Dimension(70, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(jTextFieldIntTime, gridBagConstraints);

        jButtonSetIntTime.setText("Set Int");
        jButtonSetIntTime.setToolTipText("Manually set integration");
        jButtonSetIntTime.setMaximumSize(new java.awt.Dimension(70, 20));
        jButtonSetIntTime.setMinimumSize(new java.awt.Dimension(70, 20));
        jButtonSetIntTime.setPreferredSize(new java.awt.Dimension(70, 20));
        jButtonSetIntTime.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSetIntTimeActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(jButtonSetIntTime, gridBagConstraints);

        jToggleButtonCh1.setText("CH1");
        buttonGroupChannels.add(jToggleButtonCh1);
        jToggleButtonCh1.setMaximumSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh1.setMinimumSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh1.setPreferredSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonCh1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(jToggleButtonCh1, gridBagConstraints);

        jToggleButtonCh2.setText("CH2");
        buttonGroupChannels.add(jToggleButtonCh2);
        jToggleButtonCh2.setMaximumSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh2.setMinimumSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh2.setPreferredSize(new java.awt.Dimension(57, 20));
        jToggleButtonCh2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonCh2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        getContentPane().add(jToggleButtonCh2, gridBagConstraints);

        jPanelLamp.setLayout(new java.awt.BorderLayout());

        jPanelLamp.setBackground(java.awt.Color.black);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 10;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 0);
        getContentPane().add(jPanelLamp, gridBagConstraints);

        jToggleButtonPeak1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/dsplus.gif")));
        jToggleButtonPeak1.setToolTipText("Peak D series");
        buttonGroupAcMode.add(jToggleButtonPeak1);
        jToggleButtonPeak1.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak1.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak1.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonPeak1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        getContentPane().add(jToggleButtonPeak1, gridBagConstraints);

        jToggleButtonPeak2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/csplus.gif")));
        jToggleButtonPeak2.setToolTipText("Peak C Series");
        buttonGroupAcMode.add(jToggleButtonPeak2);
        jToggleButtonPeak2.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak2.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak2.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonPeak2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        getContentPane().add(jToggleButtonPeak2, gridBagConstraints);

        jToggleButtonPeak3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/dsminus.gif")));
        jToggleButtonPeak3.setToolTipText("Peak D series");
        buttonGroupAcMode.add(jToggleButtonPeak3);
        jToggleButtonPeak3.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak3.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak3.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonPeak3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonPeak3ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        getContentPane().add(jToggleButtonPeak3, gridBagConstraints);

        jToggleButton3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/csminus.gif")));
        jToggleButton3.setToolTipText("Peak C series");
        buttonGroupAcMode.add(jToggleButton3);
        jToggleButton3.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButton3.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButton3.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton3ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        getContentPane().add(jToggleButton3, gridBagConstraints);

        jToggleButtonDC.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/dc.gif")));
        jToggleButtonDC.setToolTipText("DC Mode");
        buttonGroupAcMode.add(jToggleButtonDC);
        jToggleButtonDC.setMaximumSize(new java.awt.Dimension(32, 32));
        jToggleButtonDC.setMinimumSize(new java.awt.Dimension(32, 32));
        jToggleButtonDC.setPreferredSize(new java.awt.Dimension(32, 32));
        jToggleButtonDC.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonDCActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        getContentPane().add(jToggleButtonDC, gridBagConstraints);

        pack();
    }//GEN-END:initComponents
    
    private void jToggleButtonDCActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonDCActionPerformed
        actualMode=DC_MODE;
        measStarted=false;        // Add your handling code here:
    }//GEN-LAST:event_jToggleButtonDCActionPerformed
    
    private void jToggleButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton3ActionPerformed
        actualMode=PEAK_MODE_4;
        measStarted=false;
    }//GEN-LAST:event_jToggleButton3ActionPerformed
    
    private void jToggleButtonPeak3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonPeak3ActionPerformed
        actualMode=PEAK_MODE_2;
        measStarted=false;
    }//GEN-LAST:event_jToggleButtonPeak3ActionPerformed
    
    private void jToggleButtonPeak2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonPeak2ActionPerformed
        actualMode=PEAK_MODE_3;
        measStarted=false;
        
    }//GEN-LAST:event_jToggleButtonPeak2ActionPerformed
    
    private void jTextFieldResultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldResultActionPerformed
                                    }//GEN-LAST:event_jTextFieldResultActionPerformed
    
    private void jToggleButtonPeak1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonPeak1ActionPerformed
        actualMode=PEAK_MODE_1;
        measStarted=false;
    }//GEN-LAST:event_jToggleButtonPeak1ActionPerformed
    
    private void jButtonSetIntTimeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSetIntTimeActionPerformed
        double interval;
        try{
            interval=
            Double.valueOf(this.jTextFieldIntTime.getText()).doubleValue();
        }
        catch (NumberFormatException ne) {
            interval=10;
            
        }
        
        
        actualNumberOfSamples=(int)(interval/1000*this.samplingFreq);
        measStarted=false;
        
        
        
    }//GEN-LAST:event_jButtonSetIntTimeActionPerformed
    
    private void jToggleButtonRMSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonRMSActionPerformed
        actualMode=RMS_MODE;
        measStarted=false;
    }//GEN-LAST:event_jToggleButtonRMSActionPerformed
    
    private void jToggleButtonAVGActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonAVGActionPerformed
        actualMode=AVG_MODE;
        measStarted=false;
        
    }//GEN-LAST:event_jToggleButtonAVGActionPerformed
    
    private void jToggleButtonCh2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonCh2ActionPerformed
        actualChannel=1;
        measStarted=false;
    }//GEN-LAST:event_jToggleButtonCh2ActionPerformed
    
    private void jToggleButtonCh1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonCh1ActionPerformed
        actualChannel=0;
        measStarted=false;
    }//GEN-LAST:event_jToggleButtonCh1ActionPerformed
    
    private void jComboBoxPLCActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxPLCActionPerformed
        switch (this.jComboBoxPLC.getSelectedIndex()){
            case PLC_SPOT:
                actualNumberOfSamples=1;
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
            case PLC_001:
                actualNumberOfSamples=(int)(0.01*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
                
            case PLC_01:
                actualNumberOfSamples=(int)(0.1*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
                
            case PLC_1:
                actualNumberOfSamples=(int)(1*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
            case PLC_2:
                actualNumberOfSamples=(int)(2.0*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
            case PLC_5:
                actualNumberOfSamples=(int)(5.0*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
                
            case PLC_10:
                actualNumberOfSamples=(int)(10*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
                
            case PLC_100:
                actualNumberOfSamples=(int)(100*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(false);
                this.jButtonSetIntTime.setEnabled(false);
                measStarted=false;
                break;
            case PLC_MANUAL:
                actualNumberOfSamples=(int)(0.01*samplingFreq/lineFreq);
                this.jTextFieldIntTime.setEnabled(true);
                this.jButtonSetIntTime.setEnabled(true);
                measStarted=false;
                break;
                
            default:
        }
        
    }//GEN-LAST:event_jComboBoxPLCActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        if (!weAreInABench){
            if (this.man!=null)
                this.man.stopCapture();
            System.exit(0);
        }
        else{
            int n = JOptionPane.showConfirmDialog(
            this,
            "This panel has been created by a bench manager. \n It should not be manually destroyed \n Are you SURE you want to continue?",
            "Alert",
            JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
            if (n==JOptionPane.YES_OPTION)
                this.dispose();
        }
        
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new JFrameDmm().show();
    }
    
    public void update(java.util.Observable observable, java.lang.Object obj) {
        DecimalFormat df2 =
        (DecimalFormat)NumberFormat.getInstance();
        df2.applyPattern("#00.000");
        
        short[] dataS=(short[])obj;
        
        long r=0;
        int start=0,availSamples;
        double rd,actualPeak=-1,actualNegativePeak=1;
        if (!measStarted){
            //this is a new measurement
            ((SignalPanel)this.jPanelLamp).setState(SignalPanel.sStateGrey);
            measStarted=true;
            if ((actualMode!=RMS_MODE) & (actualMode!=AVG_MODE)){
                start=0;
            }else{
                start=(int)(Math.random()*500)*2; //500 samples *2 channels
            }
            availSamples=dataS.length/2-start/2;
            if (availSamples>=actualNumberOfSamples){
                //we have enough samples to compute the result immediately
                if (actualMode==RMS_MODE){
                    for (int i=0;i<actualNumberOfSamples;i++)
                        r+=dataS[start+actualChannel+2*i]*dataS[start+actualChannel+2*i];
                    rd=Math.sqrt((double)r/actualNumberOfSamples);
                }else if (actualMode==AVG_MODE){
                    for (int i=0;i<actualNumberOfSamples;i++){
                        if (dataS[start+actualChannel+2*i]<0)
                            r-=dataS[start+actualChannel+2*i];
                        else
                            r+=dataS[start+actualChannel+2*i];
                    }
                    rd=(double)r/actualNumberOfSamples*AVG2RMS;
                }else if (actualMode==PEAK_MODE_1){
                    //peak mode 1 is with series diode so find the peak and compute
                    //the discharge
                    for (int i=0;i<actualNumberOfSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=dataS[start+actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=dataS[start+actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                }else if (actualMode==PEAK_MODE_2){
                    //peak mode 2 is with negative series diode so find the peak and compute
                    //the discharge
                    for (int i=0;i<actualNumberOfSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[start+actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (-dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[start+actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                }else if (actualMode==PEAK_MODE_3){
                    //peak mode 3 is with parallel diode
                    //find the minimum and do an average on the signal
                    //shifted by the minimum
                    for (int i=0;i<actualNumberOfSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[start+actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[start+actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                    
                }else if (actualMode==PEAK_MODE_4){
                    //peak mode 4 is with parallel diode reversed
                    //find the minimum and do an average on the signal
                    //shifted by the minimum
                    for (int i=0;i<actualNumberOfSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[start+actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=-dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (-dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[start+actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                    
                }
                else if (actualMode==DC_MODE){
                    for (int i=0;i<actualNumberOfSamples;i++)
                        r+=dataS[start+actualChannel+2*i];
                    rd=(double)r/actualNumberOfSamples;}
                
                else
                    rd=0;
                //the result (in bits) isw in r
                if (useCalibrationFactor){
                    double actualVolt;
                    if (actualChannel==0)
                        actualVolt=rd*this.calFactorVoltAtBitCh1*1000; //mV
                    else
                        actualVolt=rd*this.calFactorVoltAtBitCh2*1000; //mV
                    
                    df2.applyPattern("#00.000");
                    String label=new String(df2.format(actualVolt));
                    this.jTextFieldResult.setText(label+" mV");
                }
                else{
                    df2.applyPattern("#####0.00");
                    String label=new String(df2.format(rd));
                    this.jTextFieldResult.setText(label +" bit");
                }
                ((SignalPanel)this.jPanelLamp).setState(SignalPanel.sStateGreen);
                measStarted=false;
            }//end if avail>actual new measurement
            else{
                //the meas cannot be finished compute partial results
                if (actualMode==RMS_MODE){
                    for (int i=0;i<availSamples;i++)
                        r+=dataS[start+actualChannel+2*i]*dataS[start+actualChannel+2*i];
                }else if (actualMode==AVG_MODE){
                    for (int i=0;i<availSamples;i++){
                        if (dataS[start+actualChannel+2*i]<0)
                            r-=dataS[start+actualChannel+2*i];
                        else
                            r+=dataS[start+actualChannel+2*i];
                    }
                }else if (actualMode==PEAK_MODE_1){
                    for (int i=0;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=dataS[start+actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    peakPos-=availSamples;
                    
                }else if (actualMode==PEAK_MODE_2){
                    for (int i=0;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[start+actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    peakPos-=availSamples;
                    
                }else if (actualMode==PEAK_MODE_3){
                    for (int i=0;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[start+actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    peakPos-=availSamples;
                }else if (actualMode==PEAK_MODE_4){
                    for (int i=0;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[start+actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=-dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    peakPos-=availSamples;
                    
                    
                }
                else if (actualMode==DC_MODE){
                    for (int i=0;i<availSamples;i++)
                        r+=dataS[start+actualChannel+2*i];}
                else
                    r=0;
                //do not show anithing bus save values for later use
                saveIntermediate=r;
                gotSamples=availSamples;
            }
        }//end if !measStarted
        else{
            //we have a running meas
            r=saveIntermediate; //reload the last values
            //can we finish ?
            availSamples=dataS.length/2;
            if (availSamples>=(actualNumberOfSamples-gotSamples)){
                //yes we can
                if (actualMode==RMS_MODE){
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++)
                        r+=dataS[actualChannel+2*i]*dataS[actualChannel+2*i];
                    
                    rd=Math.sqrt((double)r/actualNumberOfSamples);
                }else if (actualMode==AVG_MODE){
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++){
                        if (dataS[actualChannel+2*i]<0)
                            r-=dataS[actualChannel+2*i];
                        else
                            r+=dataS[actualChannel+2*i];
                    }
                    rd=(double)r/actualNumberOfSamples*AVG2RMS;
                } else if (actualMode==PEAK_MODE_1){
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[actualChannel+2*i]>actualPeak){
                            peakVal=dataS[actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                        }
                        r+=actualPeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples-gotSamples;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=dataS[actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                } else if (actualMode==PEAK_MODE_2){
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                        }
                        r+=actualPeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples-gotSamples;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (-dataS[start+actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                    
                    
                }else if (actualMode==PEAK_MODE_3){
                    //peak mode 3 is with parallel diode
                    //find the minimum and do an average on the signal
                    //shifted by the minimum
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples-gotSamples;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                    
                    
                }else if (actualMode==PEAK_MODE_4){
                    //peak mode 4 is with parallel diode reversed
                    //find the minimum and do an average on the signal
                    //shifted by the minimum
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=-dataS[start+actualChannel+2*i]-actualNegativePeak;
                    }
                    rd=(double)r/actualNumberOfSamples*PEAK2RMS*correctionFactor;
                    //take care of the peak value
                    for (int i=actualNumberOfSamples-gotSamples;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        
                        if (-dataS[start+actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[actualChannel+2*i];
                            peakPos=i;
                        }
                    }
                    peakPos-=availSamples;
                    
                }
                else if (actualMode==DC_MODE){
                    for (int i=0;i<actualNumberOfSamples-gotSamples;i++)
                        r+=dataS[actualChannel+2*i];
                    rd=(double)r/actualNumberOfSamples;}
                else
                    rd=0;
                //the result in bits is in r
                if (useCalibrationFactor){
                    double actualVolt;
                    if (actualChannel==0)
                        actualVolt=rd*this.calFactorVoltAtBitCh1*1000; //mV
                    else
                        actualVolt=rd*this.calFactorVoltAtBitCh2*1000; //mV
                    
                    df2.applyPattern("#00.000");
                    String label=new String(df2.format(actualVolt));
                    this.jTextFieldResult.setText(label+" mV");
                }
                else{
                    df2.applyPattern("#####0.00");
                    String label=new String(df2.format(rd));
                    this.jTextFieldResult.setText(label +" bit");
                }
                ((SignalPanel)this.jPanelLamp).setState(SignalPanel.sStateGreen);
                measStarted=false;
            }//end of we can finish
            else{
                //no we cannot finish so continue to process
                if (actualMode==RMS_MODE){
                    for (int i=0;i<availSamples;i++)
                        r+=dataS[actualChannel+2*i]*dataS[actualChannel+2*i];
                }else if (actualMode==AVG_MODE){
                    for (int i=0;i<availSamples;i++){
                        if (dataS[actualChannel+2*i]<0)
                            r-=dataS[actualChannel+2*i];
                        else
                            r+=dataS[actualChannel+2*i];
                    }
                }
                else if (actualMode==PEAK_MODE_1){
                    for (int i=0;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[actualChannel+2*i]>actualPeak){
                            peakVal=dataS[actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    peakPos-=availSamples;
                }else if (actualMode==PEAK_MODE_2){
                    for (int i=0;i<availSamples;i++){
                        actualPeak=(double)peakVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[actualChannel+2*i]>actualPeak){
                            peakVal=-dataS[actualChannel+2*i];
                            actualPeak=peakVal;
                            peakPos=i;
                            
                        }
                        r+=actualPeak;
                    }
                    peakPos-=availSamples;
                    
                }else if (actualMode==PEAK_MODE_3){
                    for (int i=0;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (dataS[actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=dataS[actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=dataS[actualChannel+2*i]-actualNegativePeak;
                    }
                    peakPos-=availSamples;
                    
                    
                }else if (actualMode==PEAK_MODE_4){
                    for (int i=0;i<availSamples;i++){
                        actualNegativePeak=(double)peakNegativeVal*Math.exp((peakPos-i)*tauPeak);
                        if (-dataS[actualChannel+2*i]<actualNegativePeak){
                            peakNegativeVal=-dataS[actualChannel+2*i];
                            actualNegativePeak=peakNegativeVal;
                            peakPos=i;
                            
                        }
                        r+=-dataS[actualChannel+2*i]-actualNegativePeak;
                    }
                    peakPos-=availSamples;
                    
                    
                }
                else if (actualMode==DC_MODE){
                    for (int i=0;i<availSamples;i++)
                        r+=dataS[actualChannel+2*i];}
                else
                    r=0;
                //do not show anything but save the values
                saveIntermediate=r;
                gotSamples+=availSamples;
                
            }//end of we cannot finish
        }//end of meas already started
        
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToggleButton jToggleButtonCh2;
    private javax.swing.JToggleButton jToggleButtonPeak2;
    private javax.swing.JComboBox jComboBoxPLC;
    private javax.swing.ButtonGroup buttonGroupAcMode;
    private javax.swing.JToggleButton jToggleButton3;
    private javax.swing.JToggleButton jToggleButtonPeak1;
    private javax.swing.JToggleButton jToggleButtonCh1;
    private javax.swing.JTextField jTextFieldIntTime;
    private javax.swing.JToggleButton jToggleButtonRMS;
    private javax.swing.JToggleButton jToggleButtonPeak3;
    private javax.swing.JPanel jPanelLamp;
    private javax.swing.JButton jButtonSetIntTime;
    private javax.swing.JTextField jTextFieldResult;
    private javax.swing.JToggleButton jToggleButtonDC;
    private javax.swing.JToggleButton jToggleButtonAVG;
    private javax.swing.ButtonGroup buttonGroupChannels;
    // End of variables declaration//GEN-END:variables
    
    class DmmMonitor extends TimerTask{
        //this class is used to update the status and trig lamps
        
        
        public DmmMonitor() {
        }
        public void run() {
            if (measStarted){
                //flash the lamp during the measurement
                flashOn=!flashOn;
                if (flashOn){
                    ((SignalPanel)jPanelLamp).setState(SignalPanel.sStateBlue);
                }else{
                    ((SignalPanel)jPanelLamp).setState(SignalPanel.sStateGrey);
                    
                }
                
            }
        }//method run
    }//class monitor
    
}
