/*
 *  JFrameFunctionGenerator.java
 *  Created on January 29, 2002, 1:31 PM
 *  Version 1beta1 corrected locale problem June 25 2002
 *  @author  Marco Parvis
 */

/*
    Questo e' JFrameFunctionGenerator una parte di Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
    This is JFrameFunctionGenerator part of the Instrument Training Project ITP
    Copyright (C) <Politecnico di Torino 2002> <Marco Parvis>
 
 */

/*
    Questo  programma e'  software  libero; e'  lecito redistribuirlo  o
    modificarlo secondo i termini  della Licenza Pubblica Generica GNU
    come e' pubblicata dalla Free  Software Foundation; o la versione 2
    della licenza o (a propria scelta) una versione successiva.
 
    Questo programma  e' distribuito nella  speranza che sia  utile, ma
    SENZA  ALCUNA GARANZIA;  senza  neppure la  garanzia implicita  di
    NEGOZIABILITA'  o di  APPLICABILITA' PER  UN PARTICOLARE  SCOPO.  Si
    veda la Licenza Pubblica Generica GNU per avere maggiori dettagli.
 
    Questo  programma deve  essere  distribuito assieme  ad una  copia
    della Licenza Pubblica Generica GNU;  in caso contrario, se ne puo'
    ottenere  una scrivendo  alla Free  Software Foundation,  Inc., 59
    Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
    L'autore puo' essere contattato a parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 */

/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
 
    The author can be contacted at parvis@polito.it
    Dipartimento di Elettronica Politecnico di Torino
    Corso Duca degli Abruzzi, 24, 10129 Torino Italy
 
 */


/*
 * This panel uses the output manager class to generate signals
 * Main problems are related to the single time base for two channels
 * This restrics the really genearble couples.
 * To allow a greater flexibility a long buffer is used so that several
 * Periods are generated. Periods need not to be integer, only the buffer
 * needs to contain an intger number of periods.
 * Unfortunately this solution generates jitter when square signals are used
 * (at least with the simple generation alghoritm I am using)
 * A button 'jitter free' is provided to restrict frequencies to integer, even
 * number of samples per period thus eliminationg the jitter problem
 *
 * @author  Marco Parvis
 */

package JavaScope;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.text.*;

public class JFrameFunctionGenerator extends javax.swing.JFrame implements VirtualPanel {
    
    private String panelIdString;
    private String lastSaveDir=new String(" ");
    private boolean useCalibrationFactor=false;
    private double calFactorVoltAtBitCh1=1;
    private double calFactorVoltAtBitCh2=1;
    private boolean weAreInABench=false;
    private boolean useStrictPeriods=true;
    
    private final double defAmpl=10000.0;
    private final double defFreq=1000.0;
    private final double defPh=0.0;
    private final double defAmplInVolt=0.1;
    private final int NO_ACTION=1;
    private final int START_WAVE=2;
    private final int START_CLIP=3;
    private final int STOP_WAVE=4;
    private int queuedAction=NO_ACTION;
    
    private final int MAX_BUFFER_SIZE=44100/5;
    private int[] defSetup=new int[4];;
    private double[] defAmplitude=new double[2];
    private int[] actualSetup=new int[4];
    private double[] actualAmplitude=new double[2];
    private double[] actualPhase=new double[2];
    private double[] defPhase=new double[2];
    
    private short[] ch1,ch2;
    private OutputManager outMng;
    private Vector clipList = new Vector(); //for arbitrary signals
    
    private double fr=100;
    
    
    /** Creates new form JFrameFunctionGenerator */
    public JFrameFunctionGenerator() {
        initComponents();
        initEditableFields();
        this.jComboBoxArbSignals.addItem("none");
        this.jComboBoxArbSignals.setEnabled(false);
        this.jLabelArb.setEnabled(false);
        this.jToggleButtonMuteCh1.setSelected(true);
        this.jToggleButtonMuteCh2.setSelected(true);
        this.jCheckBoxStrict.setSelected(true);
        this.jToggleButtonARB.setEnabled(false);
        this.jLabelJitter.setVisible(false);
        this.jPanelJitterCh1.setVisible(false);
        this.jPanelJitterCh2.setVisible(false);
        computeActualSetUpValues();
        defSetup=actualSetup;
        defAmplitude=actualAmplitude;
        defPhase=actualPhase;
        this.outMng=new OutputManager();
        outMng.addObserver(this);
        
        setCommandEnable(true);
        this.setTitle("Function Generator");
        weAreInABench=false;
    }
    public JFrameFunctionGenerator(OutputManager outM) {
        
        initComponents();
        this.outMng=outM;
        initEditableFields();
        this.jComboBoxArbSignals.addItem("none");
        this.jComboBoxArbSignals.setEnabled(false);
        this.jLabelArb.setEnabled(false);
        this.jToggleButtonMuteCh1.setSelected(true);
        this.jToggleButtonMuteCh2.setSelected(true);
        this.jCheckBoxStrict.setSelected(true);
        this.jToggleButtonARB.setEnabled(false);
        this.jPanelJitterCh1.setVisible(false);
        this.jPanelJitterCh2.setVisible(false);
        this.jLabelJitter.setVisible(false);
        computeActualSetUpValues();
        defSetup=actualSetup;
        defAmplitude=actualAmplitude;
        outMng.addObserver(this);
        setCommandEnable(true);
        weAreInABench=false;
        this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        
    }//of constructor
    
    private void initEditableFields(){
        DecimalFormat df2 = (DecimalFormat)NumberFormat.getInstance();
        df2.applyPattern("####0.0");
        this.jTextFieldFreq1.setText
        (new String(df2.format(defFreq)));
        this.jTextFieldFreq2.setText
        (new String(df2.format(defFreq)));
        df2.applyPattern("###0.00");
        this.jTextFieldAmplitude1.setText
        (new String(df2.format(defAmpl)));
        this.jTextFieldAmplitude2.setText
        (new String(df2.format(defAmpl)));
        this.jTextFieldPhase1.setText
        (new String(df2.format(defPh)));
        this.jTextFieldPhase2.setText
        (new String(df2.format(defPh)));
        
    }
    
    public String getSettings(){
        
        int x=this.getLocation().x;
        int y=this.getLocation().y;
        int w=this.getSize().width;
        int h=this.getSize().height;
        
        String s=panelIdString+"|Position="+String.valueOf(x)+","+String.valueOf(y)+"|Size="+String.valueOf(w)+","+String.valueOf(h)+"|";
        if (this.useCalibrationFactor)
            s=s+"UseOutputCal=1|";
        else
            s=s+"UseOutputCal=0|";
        s=s+"OutputCalFactors="+String.valueOf(this.calFactorVoltAtBitCh1)+","+String.valueOf(this.calFactorVoltAtBitCh2);
        s=s+"|LastDir="+this.lastSaveDir;
        return s;
    }
    public void applySettings(String settings){
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        StringTokenizer tokens=new StringTokenizer(settings,"|");
        while (tokens.hasMoreTokens()){
            String token=tokens.nextToken();
            //browse through recognized tokens
            if (token.indexOf("Position=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<1) x=1;
                    if (x>screenSize.width-10) x=screenSize.width-10;
                    if (y<1) y=1;
                    if (y>screenSize.height-10) y=screenSize.height-10;
                    this.setLocation(x,y);
                }catch (Throwable t){}
                
            }else if (token.indexOf("Size=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    int y=new Integer(p[1]).intValue();
                    if (x<10) x=10;
                    if (x>screenSize.width-20) x=screenSize.width-20;
                    if (y<10) y=10;
                    if (y>screenSize.height-20) y=screenSize.height-20;
                    this.setSize(x,y);
                }catch (Throwable t){}
            }else if (token.indexOf("UseOutputCal=")>=0){
                String[] p=getValues(token);
                try{
                    int x=new Integer(p[0]).intValue();
                    if (x!=0)
                        this.useCalibrationFactor=true;
                    else
                        this.useCalibrationFactor=false;
                }catch (Throwable t){}
            }else if (token.indexOf("OutputCalFactors=")>=0){
                String[] p=getValues(token);
                try{
                    double x=new Double(p[0]).doubleValue();
                    double y=new  Double(p[1]).doubleValue();
                    this.calFactorVoltAtBitCh1=x;
                    this.calFactorVoltAtBitCh2=y;
                }catch (Throwable t){}
                
            }else if (token.indexOf("LastDir=")>=0){
                String[] p=getValues(token);
                try{
                    this.lastSaveDir=p[0];
                }catch (Throwable t){}
            }
        }
        
    }
    private String[] getValues(String s){
        int peq=s.indexOf("=");
        if (peq>0){
            StringTokenizer ss=new StringTokenizer(s.substring(peq+1),",");
            if (ss.countTokens()>0){
                String[] vl=new String[ss.countTokens()];
                for (int i=0;i<vl.length;i++)
                    vl[i]=ss.nextToken();
                return vl;
            }else{
                return new String[0];
            }
            
        }else
            return new String[0];
    }
    
    public void useInputCalibration(boolean use){
    }
    public void useOutputCalibration(boolean use){
        DecimalFormat df2 = (DecimalFormat)NumberFormat.getInstance();
        this.jTextFieldFreq1.setText
        (new String(df2.format(defFreq)));
        
        
        this.useCalibrationFactor=use;
        if (use){
            df2.applyPattern("####0.0");
            this.jLabelAmpl.setText("Amp. mV");
            this.jTextFieldAmplitude1.setText
            (new String(df2.format(defAmplInVolt*100)));
            this.jTextFieldAmplitude2.setText
            (new String(df2.format(defAmplInVolt*100)));
        }
        else{
            df2.applyPattern("####0");
            this.jLabelAmpl.setText("Amp. bit");
            this.jTextFieldAmplitude1.setText
            (new String(df2.format(defAmpl)));
            this.jTextFieldAmplitude2.setText
            (new String(df2.format(defAmpl)));
        }
    }
    public void setInputCalibration(double newCalCh1,double newCalCh2){
    }
    public void setOutputCalibration(double newCalCh1,double newCalCh2){
        this.calFactorVoltAtBitCh1=newCalCh1;
        this.calFactorVoltAtBitCh2=newCalCh2;
    }
    public void setInputOffset(short newOffsetCh1, short newOffsetCh2) {
    }
    
    public void useInputOffset(boolean use) {
    }
    
    public void setBenchComponent(boolean bc){
        weAreInABench=bc;
        if (bc)
            this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        else
            this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
    }
    public void setIdString(String s){
        this.panelIdString=s;
        this.setTitle(s);
    }
    public String getIdString(){
        return this.panelIdString;
    }
    
    public void setInputManager(InputManager om){
    }
    public void setOutputManager(OutputManager om){
        this.outMng=om;
        outMng.addObserver(this);
    }
    
    
    
    
    public void update(Observable o, Object arg){
        int resp=((Integer)arg).intValue();
        if (resp==outMng.OUT_CHUNK_DONE){
            //do nothing the chunk is automatically reloaded
            /*
             int[] setUp=new int[2];
            setUp[0]=actualSetup[0]; //samples
            setUp[1]=actualSetup[1]; //periods
            setUp[1]=setUp[1]+(int)fr;
            fr=fr+1;
            if (fr>1000)
                fr=0;
            ch1=computeSinSamples(setUp,21000,0);
            ch2=computeSinSamples(setUp,21000,0);
            if (!this.outMng.updateWaveforms(ch1,ch2))
                System.out.println("Error setting waveforms");
            //this message could be used to change the signal on the fly
             */
        }else if (resp==outMng.OUT_STARTED | resp==outMng.OUT_STOPPED){
            switch (queuedAction){
                case NO_ACTION:
                    return;
                case START_WAVE:
                    outMng.startWav();
                    queuedAction=NO_ACTION;
                    break;
                case START_CLIP:
                    if (!outMng.startClip()){
                        this.jToggleButtonARB.setSelected(false);
                        setCommandEnable(true);
                        startGenerator();
                    }
                    queuedAction=NO_ACTION;
                    break;
                case STOP_WAVE:
                    outMng.stopPlay();
                    queuedAction=NO_ACTION;
                    break;
            }
        }else if (resp==outMng.OUT_ERROR){
            System.out.println("FUNC GEN: error from out manager");
            queuedAction=NO_ACTION;
        }
        
    }//of update
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupSignals = new javax.swing.ButtonGroup();
        buttonGroupSignalsCh2 = new javax.swing.ButtonGroup();
        buttonGroupSignalsCh1 = new javax.swing.ButtonGroup();
        jPanelBase = new javax.swing.JPanel();
        jPanelCharact = new javax.swing.JPanel();
        jTextFieldFreq1 = new javax.swing.JTextField();
        jTextFieldFreq2 = new javax.swing.JTextField();
        jTextFieldAmplitude1 = new javax.swing.JTextField();
        jTextFieldAmplitude2 = new javax.swing.JTextField();
        jLabelFreq = new javax.swing.JLabel();
        jLabelAmpl = new javax.swing.JLabel();
        jLabelCh1 = new javax.swing.JLabel();
        jLabelCh2 = new javax.swing.JLabel();
        jTextFieldPhase1 = new javax.swing.JTextField();
        jTextFieldPhase2 = new javax.swing.JTextField();
        jLabelPhase = new javax.swing.JLabel();
        jLabelArb = new javax.swing.JLabel();
        jComboBoxArbSignals = new javax.swing.JComboBox();
        jToggleButtonSinCh1 = new javax.swing.JToggleButton();
        jToggleButtonSqCh1 = new javax.swing.JToggleButton();
        jToggleButtonTriCh1 = new javax.swing.JToggleButton();
        jToggleButtonSinCh2 = new javax.swing.JToggleButton();
        jToggleButtonSqCh2 = new javax.swing.JToggleButton();
        jToggleButtonTriCh2 = new javax.swing.JToggleButton();
        jToggleButtonMuteCh1 = new javax.swing.JToggleButton();
        jToggleButtonMuteCh2 = new javax.swing.JToggleButton();
        jLabelFunction = new javax.swing.JLabel();
        jToggleButtonARB = new javax.swing.JToggleButton();
        jCheckBoxStrict = new javax.swing.JCheckBox();
        jPanelJitterCh1 = new SignalPanel();
        jPanelJitterCh2 = new SignalPanel();
        jLabelJitter = new javax.swing.JLabel();
        jLabelFiller = new javax.swing.JLabel();
        jToolBarWave = new javax.swing.JToolBar();
        jButtonLoad = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jPanelBase.setLayout(new java.awt.GridBagLayout());

        jPanelBase.setBorder(new javax.swing.border.EtchedBorder());
        jPanelBase.setPreferredSize(new java.awt.Dimension(600, 112));
        jPanelCharact.setLayout(new java.awt.GridBagLayout());

        jPanelCharact.setBorder(new javax.swing.border.TitledBorder("Characteristics"));
        jPanelCharact.setMinimumSize(new java.awt.Dimension(600, 118));
        jPanelCharact.setPreferredSize(new java.awt.Dimension(500, 120));
        jTextFieldFreq1.setText("freq1");
        jTextFieldFreq1.setToolTipText("Yellow color requires update");
        jTextFieldFreq1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldFreq1ActionPerformed(evt);
            }
        });
        jTextFieldFreq1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldFreq1KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 50;
        jPanelCharact.add(jTextFieldFreq1, gridBagConstraints);

        jTextFieldFreq2.setText("freq2");
        jTextFieldFreq2.setToolTipText("Yellow color requires update");
        jTextFieldFreq2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldFreq2ActionPerformed(evt);
            }
        });
        jTextFieldFreq2.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldFreq2KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 20;
        jPanelCharact.add(jTextFieldFreq2, gridBagConstraints);

        jTextFieldAmplitude1.setText("ampli1");
        jTextFieldAmplitude1.setToolTipText("Yellow color requires update");
        jTextFieldAmplitude1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldAmplitude1KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 50;
        jPanelCharact.add(jTextFieldAmplitude1, gridBagConstraints);

        jTextFieldAmplitude2.setText("ampli2");
        jTextFieldAmplitude2.setToolTipText("Yellow color requires update");
        jTextFieldAmplitude2.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldAmplitude2KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jTextFieldAmplitude2, gridBagConstraints);

        jLabelFreq.setText("Freq");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jLabelFreq, gridBagConstraints);

        jLabelAmpl.setText("Amp. bit");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jLabelAmpl, gridBagConstraints);

        jLabelCh1.setText("CH1  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jLabelCh1, gridBagConstraints);

        jLabelCh2.setText("CH2");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jLabelCh2, gridBagConstraints);

        jTextFieldPhase1.setText("phase1");
        jTextFieldPhase1.setToolTipText("Yellow color requires update");
        jTextFieldPhase1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldPhase1KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 50;
        jPanelCharact.add(jTextFieldPhase1, gridBagConstraints);

        jTextFieldPhase2.setText("phase2");
        jTextFieldPhase2.setToolTipText("Yellow color requires update");
        jTextFieldPhase2.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldPhase2KeyTyped(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jTextFieldPhase2, gridBagConstraints);

        jLabelPhase.setText("Phase");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jLabelPhase, gridBagConstraints);

        jLabelArb.setText("Arb signals");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanelCharact.add(jLabelArb, gridBagConstraints);

        jComboBoxArbSignals.setToolTipText("Available arbitrary signals");
        jComboBoxArbSignals.setPreferredSize(new java.awt.Dimension(31, 20));
        jComboBoxArbSignals.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxArbSignalsActionPerformed(evt);
            }
        });
        jComboBoxArbSignals.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                jComboBoxArbSignalsCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
            }
        });
        jComboBoxArbSignals.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBoxArbSignalsItemStateChanged(evt);
            }
        });
        jComboBoxArbSignals.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jComboBoxArbSignalsMouseClicked(evt);
            }
        });
        jComboBoxArbSignals.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                jComboBoxArbSignalsPropertyChange(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelCharact.add(jComboBoxArbSignals, gridBagConstraints);

        jToggleButtonSinCh1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/sin.gif")));
        jToggleButtonSinCh1.setToolTipText("Sinusoidal Signal");
        buttonGroupSignalsCh1.add(jToggleButtonSinCh1);
        jToggleButtonSinCh1.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonSinCh1.setPreferredSize(new java.awt.Dimension(40, 24));
        jToggleButtonSinCh1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonSinCh1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        jPanelCharact.add(jToggleButtonSinCh1, gridBagConstraints);

        jToggleButtonSqCh1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/square.gif")));
        jToggleButtonSqCh1.setToolTipText("Square Signal");
        buttonGroupSignalsCh1.add(jToggleButtonSqCh1);
        jToggleButtonSqCh1.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonSqCh1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonSqCh1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 1;
        jPanelCharact.add(jToggleButtonSqCh1, gridBagConstraints);

        jToggleButtonTriCh1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/triangle.gif")));
        jToggleButtonTriCh1.setToolTipText("Triangle Signal");
        buttonGroupSignalsCh1.add(jToggleButtonTriCh1);
        jToggleButtonTriCh1.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonTriCh1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonTriCh1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 1;
        jPanelCharact.add(jToggleButtonTriCh1, gridBagConstraints);

        jToggleButtonSinCh2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/sin.gif")));
        jToggleButtonSinCh2.setToolTipText("Sinusoidal Signal");
        buttonGroupSignalsCh2.add(jToggleButtonSinCh2);
        jToggleButtonSinCh2.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonSinCh2.setPreferredSize(new java.awt.Dimension(40, 24));
        jToggleButtonSinCh2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonSinCh2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 2;
        jPanelCharact.add(jToggleButtonSinCh2, gridBagConstraints);

        jToggleButtonSqCh2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/square.gif")));
        jToggleButtonSqCh2.setToolTipText("Square Signal");
        buttonGroupSignalsCh2.add(jToggleButtonSqCh2);
        jToggleButtonSqCh2.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonSqCh2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonSqCh2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 2;
        jPanelCharact.add(jToggleButtonSqCh2, gridBagConstraints);

        jToggleButtonTriCh2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/triangle.gif")));
        jToggleButtonTriCh2.setToolTipText("Triangle Signal");
        buttonGroupSignalsCh2.add(jToggleButtonTriCh2);
        jToggleButtonTriCh2.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonTriCh2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonTriCh2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 2;
        jPanelCharact.add(jToggleButtonTriCh2, gridBagConstraints);

        jToggleButtonMuteCh1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/mute.gif")));
        jToggleButtonMuteCh1.setToolTipText("Mute");
        buttonGroupSignalsCh1.add(jToggleButtonMuteCh1);
        jToggleButtonMuteCh1.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonMuteCh1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonMuteCh1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 1;
        jPanelCharact.add(jToggleButtonMuteCh1, gridBagConstraints);

        jToggleButtonMuteCh2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/mute.gif")));
        jToggleButtonMuteCh2.setToolTipText("Mute");
        buttonGroupSignalsCh2.add(jToggleButtonMuteCh2);
        jToggleButtonMuteCh2.setMinimumSize(new java.awt.Dimension(40, 24));
        jToggleButtonMuteCh2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonMuteCh2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 2;
        jPanelCharact.add(jToggleButtonMuteCh2, gridBagConstraints);

        jLabelFunction.setText("Function");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanelCharact.add(jLabelFunction, gridBagConstraints);

        jToggleButtonARB.setText("ARB");
        jToggleButtonARB.setToolTipText("Use arbitrary signals");
        jToggleButtonARB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonARBActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanelCharact.add(jToggleButtonARB, gridBagConstraints);

        jCheckBoxStrict.setText("Jitter free");
        jCheckBoxStrict.setToolTipText("Insure jitter free signals");
        jCheckBoxStrict.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxStrictActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        jPanelCharact.add(jCheckBoxStrict, gridBagConstraints);

        jPanelJitterCh1.setToolTipText("Jitter indicator");
        jPanelJitterCh1.setMinimumSize(new java.awt.Dimension(12, 12));
        jPanelJitterCh1.setPreferredSize(new java.awt.Dimension(12, 12));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        gridBagConstraints.gridy = 1;
        jPanelCharact.add(jPanelJitterCh1, gridBagConstraints);

        jPanelJitterCh2.setToolTipText("Jitter indicator");
        jPanelJitterCh2.setMinimumSize(new java.awt.Dimension(12, 12));
        jPanelJitterCh2.setPreferredSize(new java.awt.Dimension(12, 12));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        gridBagConstraints.gridy = 2;
        jPanelCharact.add(jPanelJitterCh2, gridBagConstraints);

        jLabelJitter.setText("Jitter");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        gridBagConstraints.gridy = 0;
        jPanelCharact.add(jLabelJitter, gridBagConstraints);

        jLabelFiller.setMaximumSize(new java.awt.Dimension(36, 0));
        jLabelFiller.setMinimumSize(new java.awt.Dimension(36, 0));
        jLabelFiller.setPreferredSize(new java.awt.Dimension(36, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        gridBagConstraints.gridy = 3;
        jPanelCharact.add(jLabelFiller, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(2, 0, 0, 0);
        jPanelBase.add(jPanelCharact, gridBagConstraints);

        getContentPane().add(jPanelBase, java.awt.BorderLayout.CENTER);

        jButtonLoad.setIcon(new javax.swing.ImageIcon(getClass().getResource("/JavaScope/resources/open.gif")));
        jButtonLoad.setToolTipText("Load Arb signals");
        jButtonLoad.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonLoad.setMinimumSize(new java.awt.Dimension(20, 24));
        jButtonLoad.setPreferredSize(new java.awt.Dimension(20, 24));
        jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonLoadActionPerformed(evt);
            }
        });

        jToolBarWave.add(jButtonLoad);

        getContentPane().add(jToolBarWave, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents
    
    private void jTextFieldFreq2KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldFreq2KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh2.isSelected())
            this.jTextFieldFreq2.setBackground(Color.yellow);
        
    }//GEN-LAST:event_jTextFieldFreq2KeyTyped
    
    private void jTextFieldPhase2KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldPhase2KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh2.isSelected())
            this.jTextFieldPhase2.setBackground(Color.yellow);
    }//GEN-LAST:event_jTextFieldPhase2KeyTyped
    
    private void jTextFieldPhase1KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldPhase1KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh1.isSelected())
            this.jTextFieldPhase1.setBackground(Color.yellow);
    }//GEN-LAST:event_jTextFieldPhase1KeyTyped
    
    private void jTextFieldAmplitude2KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldAmplitude2KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh2.isSelected())
            this.jTextFieldAmplitude2.setBackground(Color.yellow);
    }//GEN-LAST:event_jTextFieldAmplitude2KeyTyped
    
    private void jTextFieldAmplitude1KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldAmplitude1KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh1.isSelected())
            this.jTextFieldAmplitude1.setBackground(Color.yellow);
    }//GEN-LAST:event_jTextFieldAmplitude1KeyTyped
    
    private void jTextFieldFreq2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldFreq2ActionPerformed
                                                            }//GEN-LAST:event_jTextFieldFreq2ActionPerformed
    
    private void jTextFieldFreq1KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldFreq1KeyTyped
        //set background to yellow to remind user that the change is not
        //effective until a function is selected
        if (!this.jToggleButtonMuteCh1.isSelected())
            this.jTextFieldFreq1.setBackground(Color.yellow);
    }//GEN-LAST:event_jTextFieldFreq1KeyTyped
    
    private void jTextFieldFreq1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldFreq1ActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_jTextFieldFreq1ActionPerformed
    
    private void jCheckBoxStrictActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxStrictActionPerformed
        if (this.jCheckBoxStrict.isSelected()){
            this.useStrictPeriods=true;
            this.jPanelJitterCh1.setVisible(false);
            this.jPanelJitterCh2.setVisible(false);
            this.jLabelJitter.setVisible(false);
            
        }else{
            this.useStrictPeriods=false;
            this.jPanelJitterCh1.setVisible(true);
            this.jPanelJitterCh2.setVisible(true);
            this.jLabelJitter.setVisible(true);
        }
        if (!outMng.isStopped())
            startGenerator();
        
    }//GEN-LAST:event_jCheckBoxStrictActionPerformed
    
    private void jToggleButtonARBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonARBActionPerformed
        if (this.jToggleButtonARB.isSelected())
            setCommandEnable(false);
        else
            setCommandEnable(true);
        
        startGenerator();
    }//GEN-LAST:event_jToggleButtonARBActionPerformed
    
    private void jToggleButtonMuteCh2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonMuteCh2ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonMuteCh2ActionPerformed
    
    private void jToggleButtonMuteCh1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonMuteCh1ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonMuteCh1ActionPerformed
    
    private void jToggleButtonTriCh2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonTriCh2ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonTriCh2ActionPerformed
    
    private void jToggleButtonTriCh1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonTriCh1ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonTriCh1ActionPerformed
    
    private void jToggleButtonSqCh2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSqCh2ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSqCh2ActionPerformed
    
    private void jToggleButtonSqCh1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSqCh1ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSqCh1ActionPerformed
    
    private void jToggleButtonSinCh2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSinCh2ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSinCh2ActionPerformed
    
    private void jToggleButtonSinCh1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSinCh1ActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSinCh1ActionPerformed
    
    private void jComboBoxArbSignalsCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_jComboBoxArbSignalsCaretPositionChanged
        // Add your handling code here:
        
    }//GEN-LAST:event_jComboBoxArbSignalsCaretPositionChanged
    
    private void jComboBoxArbSignalsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jComboBoxArbSignalsMouseClicked
        // Add your handling code here:
        
    }//GEN-LAST:event_jComboBoxArbSignalsMouseClicked
    
    private void jComboBoxArbSignalsPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_jComboBoxArbSignalsPropertyChange
        // Add your handling code here:
        
    }//GEN-LAST:event_jComboBoxArbSignalsPropertyChange
    
    private void jComboBoxArbSignalsItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jComboBoxArbSignalsItemStateChanged
        
        if (this.jToggleButtonARB.isSelected()){
            JComboBox cb = (JComboBox)evt.getSource();
            if (!cb.isEnabled())
                return;
            if (evt.getStateChange()!=evt.SELECTED)
                return;
            if (clipList.isEmpty())
                return;
            int id=cb.getSelectedIndex();
            outMng.setClip((File)clipList.get(id));
            startClip();
        }
        
    }//GEN-LAST:event_jComboBoxArbSignalsItemStateChanged
    
    private void jComboBoxArbSignalsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxArbSignalsActionPerformed
        // Add your handling code here:
        
        
    }//GEN-LAST:event_jComboBoxArbSignalsActionPerformed
    
    private void jButtonLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonLoadActionPerformed
        // load a clip list
        JFileChooser fc=new JFileChooser();
        fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File f) {
                if (f.isDirectory()) {
                    return true;
                }
                String name = f.getName();
                if (name.endsWith(".wav")) {
                    return true;
                }
                return false;
            }
            public String getDescription() {
                return "Wav files  (*.wav) ";
            }
        });
        fc.setCurrentDirectory(new File(lastSaveDir));
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
            loadClipList(fc.getCurrentDirectory());
            this.jComboBoxArbSignals.setEnabled(true);
            this.jLabelArb.setEnabled(true);
            this.jToggleButtonARB.setEnabled(true);
            this.lastSaveDir=new String(fc.getCurrentDirectory().toString());
            //System.out.println(this.lastSaveDir);
        }
    }//GEN-LAST:event_jButtonLoadActionPerformed
                                private void jToggleButtonMuteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonMuteActionPerformed
                                    setCommandEnable(true);
                                    outMng.stopPlay();
    }//GEN-LAST:event_jToggleButtonMuteActionPerformed
                                
    private void jToggleButtonSquareActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSquareActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSquareActionPerformed
    
    private void jToggleButtonSinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSinActionPerformed
        startGenerator();
    }//GEN-LAST:event_jToggleButtonSinActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        //prevent a brutal exit if the panel is part of a bench
        if (!weAreInABench){
            if (this.outMng!=null)
                this.outMng.stopPlay();
            System.exit(0);
        }
        else{
            int n = JOptionPane.showConfirmDialog(
            this,
            "This panel has been created by a bench manager. \n It should not be manually destroyed \n Are you SURE you want to continue?",
            "Alert",
            JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
            if (n==JOptionPane.YES_OPTION)
                this.dispose();
        }
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        
        new JFrameFunctionGenerator().show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupSignals;
    private javax.swing.ButtonGroup buttonGroupSignalsCh1;
    private javax.swing.ButtonGroup buttonGroupSignalsCh2;
    private javax.swing.JButton jButtonLoad;
    private javax.swing.JCheckBox jCheckBoxStrict;
    private javax.swing.JComboBox jComboBoxArbSignals;
    private javax.swing.JLabel jLabelAmpl;
    private javax.swing.JLabel jLabelArb;
    private javax.swing.JLabel jLabelCh1;
    private javax.swing.JLabel jLabelCh2;
    private javax.swing.JLabel jLabelFiller;
    private javax.swing.JLabel jLabelFreq;
    private javax.swing.JLabel jLabelFunction;
    private javax.swing.JLabel jLabelJitter;
    private javax.swing.JLabel jLabelPhase;
    private javax.swing.JPanel jPanelBase;
    private javax.swing.JPanel jPanelCharact;
    private javax.swing.JPanel jPanelJitterCh1;
    private javax.swing.JPanel jPanelJitterCh2;
    private javax.swing.JTextField jTextFieldAmplitude1;
    private javax.swing.JTextField jTextFieldAmplitude2;
    private javax.swing.JTextField jTextFieldFreq1;
    private javax.swing.JTextField jTextFieldFreq2;
    private javax.swing.JTextField jTextFieldPhase1;
    private javax.swing.JTextField jTextFieldPhase2;
    private javax.swing.JToggleButton jToggleButtonARB;
    private javax.swing.JToggleButton jToggleButtonMuteCh1;
    private javax.swing.JToggleButton jToggleButtonMuteCh2;
    private javax.swing.JToggleButton jToggleButtonSinCh1;
    private javax.swing.JToggleButton jToggleButtonSinCh2;
    private javax.swing.JToggleButton jToggleButtonSqCh1;
    private javax.swing.JToggleButton jToggleButtonSqCh2;
    private javax.swing.JToggleButton jToggleButtonTriCh1;
    private javax.swing.JToggleButton jToggleButtonTriCh2;
    private javax.swing.JToolBar jToolBarWave;
    // End of variables declaration//GEN-END:variables
    
    
    private void setCommandEnable(boolean enable){
        this.jLabelCh1.setEnabled(enable);
        this.jLabelCh2.setEnabled(enable);
        this.jTextFieldAmplitude1.setEnabled(enable);
        this.jTextFieldAmplitude2.setEnabled(enable);
        this.jTextFieldFreq1.setEnabled(enable);
        this.jTextFieldFreq2.setEnabled(enable);
        this.jTextFieldPhase1.setEnabled(enable);
        this.jTextFieldPhase2.setEnabled(enable);
        this.jToggleButtonSinCh1.setEnabled(enable);
        this.jToggleButtonSqCh1.setEnabled(enable);
        this.jToggleButtonTriCh1.setEnabled(enable);
        this.jToggleButtonMuteCh1.setEnabled(enable);
        this.jToggleButtonSinCh2.setEnabled(enable);
        this.jToggleButtonSqCh2.setEnabled(enable);
        this.jToggleButtonTriCh2.setEnabled(enable);
        this.jToggleButtonMuteCh2.setEnabled(enable);
        this.jLabelFreq.setEnabled(enable);
        this.jLabelAmpl.setEnabled(enable);
        this.jLabelPhase.setEnabled(enable);
        this.jCheckBoxStrict.setEnabled(enable);
        this.jPanelJitterCh1.setEnabled(enable);
        this.jPanelJitterCh2.setEnabled(enable);
        this.jLabelJitter.setEnabled(enable);
        this.jLabelFunction.setEnabled(enable);
        if (!enable){
            ((SignalPanel)this.jPanelJitterCh1).setState(SignalPanel.sStateGrey);
            ((SignalPanel)this.jPanelJitterCh2).setState(SignalPanel.sStateGrey);
        }
    }
    
    public void loadClipList(File file) {
        
        try {
            this.jComboBoxArbSignals.removeAllItems();
            if (file != null && file.isDirectory()) {
                String files[] = file.list();
                for (int i = 0; i < files.length; i++) {
                    File leafFile = new File(file.getAbsolutePath(), files[i]);
                    if (leafFile.isDirectory()) {
                        loadClipList(leafFile);
                    } else {
                        addClip(leafFile);
                    }
                }
            } else if (file != null && file.exists()) {
                addClip(file);
            }
        } catch (SecurityException ex) {
            System.out.println(ex.toString());
        } catch (Exception ex) {
            System.out.println(ex.toString());
        }
    }
    
    
    private void addClip(File file) {
        String s = file.getName();
        if (s.endsWith(".wav") ) {
            this.jComboBoxArbSignals.addItem(file.getName());
            clipList.add(file);
        }
    }
    
    
    private void startClip(){
        if (outMng.isStopped()){
            if (!this.outMng.startClip()){
                this.jToggleButtonARB.setSelected(false);
                setCommandEnable(true);
                startGenerator();
            }
            
            
        }else{
            outMng.stopPlay();
            queuedAction=START_CLIP;
        }
    }//of startClip
    private void startGenerator(){
        //reset yellow backgrounds
        this.jTextFieldAmplitude1.setBackground(Color.white);
        this.jTextFieldAmplitude2.setBackground(Color.white);
        this.jTextFieldFreq1.setBackground(Color.white);
        this.jTextFieldFreq2.setBackground(Color.white);
        this.jTextFieldPhase1.setBackground(Color.white);
        this.jTextFieldPhase2.setBackground(Color.white);
        
        
        
        if (this.jToggleButtonARB.isSelected()){
            if (clipList.isEmpty())
                return;
            int id=this.jComboBoxArbSignals.getSelectedIndex();
            outMng.setClip((File)clipList.get(id));
            startClip();
            return;
        }
        //firstly compute the setup values in terms of number of samples
        //in the buffer and number of periods
        computeActualSetUpValues();
        int[] setUp=new int[2];
        setUp[0]=actualSetup[0];
        setUp[1]=actualSetup[1];
        //-------------------------------------------
        if (this.jToggleButtonSinCh1.isSelected()){
            ch1=computeSinSamples(setUp,actualAmplitude[0],actualPhase[0]);
        }else if (this.jToggleButtonSqCh1.isSelected()){
            ch1=computeSquareSamples(setUp,actualAmplitude[0],actualPhase[0]);
        }else if (this.jToggleButtonTriCh1.isSelected()){
            ch1=computeTriangleSamples(setUp,actualAmplitude[0],actualPhase[0]);
        }else if (this.jToggleButtonMuteCh1.isSelected()){
            ch1= new short[setUp[0]];
            
        }   else {
            ch1=computeSinSamples(setUp,actualAmplitude[0],actualPhase[0]);
        }
        setUp[1]=actualSetup[3];
        if (this.jToggleButtonSinCh2.isSelected()){
            ch2=computeSinSamples(setUp,actualAmplitude[1],actualPhase[1]);
        }else if (this.jToggleButtonSqCh2.isSelected()){
            ch2=computeSquareSamples(setUp,actualAmplitude[1],actualPhase[1]);
        }else if (this.jToggleButtonTriCh2.isSelected()){
            ch2=computeTriangleSamples(setUp,actualAmplitude[1],actualPhase[1]);
        }else if (this.jToggleButtonMuteCh2.isSelected()){
            ch2= new short[setUp[0]];
        }else{
            ch2=computeSinSamples(setUp,actualAmplitude[1],actualPhase[1]);
        }
        
        if (!this.outMng.setWaveforms(ch1,ch2))
            System.out.println("Error setting waveforms");
        else{
            if (outMng.isStopped()){
                //the manager was stopped start it
                this.outMng.startWav();
            }else{
                //the manager was playing stop it and queue a new start
                outMng.stopPlay();
                queuedAction=START_WAVE;
            }
        }
    }
    private double parseAndValidate(JTextField v, double defValue){
        String st="";
        
        try{
            DecimalFormat df2 =
            (DecimalFormat)NumberFormat.getInstance();
            st=v.getText();
            Number n= df2.parse(st);
            return n.doubleValue();
        }
        catch(Throwable t){
            System.out.println("String "+st+" is not a correct number for this locale !");
            return defValue;
        }
    }
    
    
    private void computeActualSetUpValues(){
        double amp1,amp2,freq1,freq2,phase1,phase2;
        int[] doubleSetup;
        DecimalFormat df2 =
        (DecimalFormat)NumberFormat.getInstance();
        amp1=parseAndValidate(this.jTextFieldAmplitude1, defAmpl);
        amp2=parseAndValidate(this.jTextFieldAmplitude2, defAmpl);
        freq1=parseAndValidate(this.jTextFieldFreq1, defFreq);
        freq2=parseAndValidate(this.jTextFieldFreq2, defFreq);
        phase1=parseAndValidate(this.jTextFieldPhase1, defPh);
        phase2=parseAndValidate(this.jTextFieldPhase2, defPh);
        
        
        //do some checks
        if (freq1<5)
            freq1=5;
        else if (freq1>20000)
            freq1=20000;
        if (freq2<5)
            freq2=5;
        else if (freq2>20000)
            freq2=20000;
        
        if (phase1>359)
            phase1=359;
        else if (phase1<0)
            phase1=0;
        if (phase2>359)
            phase2=359;
        else if (phase2<0)
            phase2=0;
        if (useCalibrationFactor){
            actualAmplitude[0]=amp1/calFactorVoltAtBitCh1/100.0;
            actualAmplitude[1]=amp2/calFactorVoltAtBitCh1/100.0;
            
        }else{
            actualAmplitude[0]=amp1;
            actualAmplitude[1]=amp2;
        }
        for (int i=0;i<2;i++)
            if (actualAmplitude[i]>Short.MAX_VALUE)
                actualAmplitude[i]=Short.MAX_VALUE;
            else if (actualAmplitude[i]<Short.MIN_VALUE)
                actualAmplitude[i]=Short.MIN_VALUE;
        
        if (this.useStrictPeriods){
            actualSetup= computeConstrainedCompatibleSamplesAndPeriods(freq1,freq2);
            if (this.jToggleButtonSqCh1.isSelected()){
                //if the signal is square the phase must be an integer
                //number
                int samplePerPeriod=actualSetup[0]/actualSetup[1];
                int p=(int)Math.round(samplePerPeriod/360.0*phase1);
                actualPhase[0]=360.0*p/samplePerPeriod;
            }else
                actualPhase[0]=phase1;
            if (this.jToggleButtonSqCh2.isSelected()){
                int samplePerPeriod=actualSetup[2]/actualSetup[3];
                int p=(int)Math.round(samplePerPeriod/360.0*phase2);
                actualPhase[1]=360.0*p/samplePerPeriod;
            }else
                actualPhase[1]=phase2;
        }            else{
            actualSetup= computeCompatibleSamplesAndPeriods(freq1,freq2);
            
            actualPhase[0]=phase1;
            actualPhase[1]=phase2;
        }
        //now update the panels to reflect the computed values
        
        df2.applyPattern("#####0.000");
        
        int[] setUp=new int[2];
        setUp[0]=actualSetup[0];
        setUp[1]=actualSetup[1];
        //System.out.println("Setup 0  "+setUp[0]+"   1:"+setUp[1]);
        df2.applyPattern("####0.0");
        this.jTextFieldFreq1.setText
        (new String(df2.format(computeRealFrequency(setUp))));
        setUp[1]=actualSetup[3];
        this.jTextFieldFreq2.setText
        (new String(df2.format(computeRealFrequency(setUp))));
        if (useCalibrationFactor){
            df2.applyPattern("###0.00");
            
            jTextFieldAmplitude1.setText
            (new String(df2.format(actualAmplitude[0]*calFactorVoltAtBitCh1*100)));
            jTextFieldAmplitude2.setText
            (new String(df2.format(actualAmplitude[1]*calFactorVoltAtBitCh1*100)));
        }
        else{
            df2.applyPattern("#####0");
            jTextFieldAmplitude1.setText
            (new String(df2.format(actualAmplitude[0])));
            jTextFieldAmplitude2.setText
            (new String(df2.format(actualAmplitude[1])));
            
        }
        df2.applyPattern("###0.00");
        this.jTextFieldPhase1.setText(new String(df2.format(actualPhase[0])));
        this.jTextFieldPhase2.setText(new String(df2.format(actualPhase[1])));
        
        if ((actualSetup[0]%actualSetup[1]!=0) & this.jToggleButtonSqCh1.isSelected())
            ((SignalPanel)this.jPanelJitterCh1).setState(SignalPanel.sStateRed);
        else
            ((SignalPanel)this.jPanelJitterCh1).setState(SignalPanel.sStateGreen);
        
        
        if ((actualSetup[2]%actualSetup[3]!=0) & this.jToggleButtonSqCh2.isSelected())
            ((SignalPanel)this.jPanelJitterCh2).setState(SignalPanel.sStateRed);
        else
            ((SignalPanel)this.jPanelJitterCh2).setState(SignalPanel.sStateGreen);
        
        
        
    }
    private int[] computeCompatibleSamplesAndPeriods(double freq1,double freq2){
        //we must found a two couples of values use the first channel as master
        //this is the jitter version
        
        //compute the number of required sample per period
        double nExactSamplesPerPeriod=44100/freq1;
        //compute the number of periods that fit in the buffer
        int nPeriods=(int)Math.floor(MAX_BUFFER_SIZE/nExactSamplesPerPeriod);
        //compute the number of samples corresponding to the periods
        int nSamples=(int)(nPeriods*nExactSamplesPerPeriod);
        //make sure the number os even
        if ((nSamples%2)!=0)
            nSamples+=1;
        //now within the same number of samples try to fit freq2
        double nExactSamplesPerPeriod2=44100/freq2;
        int nPeriods2=(int)Math.round((nSamples/nExactSamplesPerPeriod2));
        
        int[] x=new int[4];
        x[0]=nSamples;
        x[1]=nPeriods;
        x[2]=nSamples;
        x[3]=nPeriods2;
        
        return x;
    }
    
    
    private int[] computeConstrainedCompatibleSamplesAndPeriods(double freq1,double freq2){
        //we must found two couples of values we use the first channel as master
        //constrained to an integer number of samples per period
        //constrained to an even number of samples per period
        
        //compute the number of required samples for both signals
        //make sure they are even
        int nSamplesPerPeriod1=(int)Math.round(44100/freq1);
        if (nSamplesPerPeriod1%2!=0)
            nSamplesPerPeriod1+=1;
        int nSamplesPerPeriod2=(int)Math.round(44100/freq2);
        if (nSamplesPerPeriod2%2!=0)
            nSamplesPerPeriod2+=1;
        //now look for the shortest buffer that contains
        //an integer number of periods of both signals
        int ts=mcm(nSamplesPerPeriod1,nSamplesPerPeriod2);
        if (ts>MAX_BUFFER_SIZE)
            //the buffer is too big try changing periods to fit them
            //within throws buffer
            // we try -4 -2 +2 +4 samples on both signals
            for (int j=0;j<3;j++){
                for (int i=0;i<3;i++){
                    ts=mcm(nSamplesPerPeriod1+2*j,nSamplesPerPeriod2+2*i);
                    if (ts<MAX_BUFFER_SIZE){
                        nSamplesPerPeriod1+=2*j;
                        nSamplesPerPeriod2+=2*i;
                        break;
                    }
                    ts=mcm(nSamplesPerPeriod1,nSamplesPerPeriod2-2*i);
                    if (ts<MAX_BUFFER_SIZE){
                        nSamplesPerPeriod1+=2*j;
                        nSamplesPerPeriod2-=2*i;
                        break;
                    }
                    ts=mcm(nSamplesPerPeriod1-2*j,nSamplesPerPeriod2+2*i);
                    if (ts<MAX_BUFFER_SIZE){
                        nSamplesPerPeriod1-=2*j;
                        nSamplesPerPeriod2+=2*i;
                        break;
                    }
                    ts=mcm(nSamplesPerPeriod1,nSamplesPerPeriod2-2*i);
                    if (ts<MAX_BUFFER_SIZE){
                        nSamplesPerPeriod1-=2*j;
                        nSamplesPerPeriod2-=2*i;
                        break;
                    }
                    
                }
            }
        //if the buffer is too short enlarge it
        int expander=MAX_BUFFER_SIZE/ts;
        if (expander>1)
            ts=ts*expander;
        //System.out.println("Using "+ts+" samples");
        int nSamples=ts;
        int nPeriods1=ts/nSamplesPerPeriod1;
        int nPeriods2=ts/nSamplesPerPeriod2;
        int[] x=new int[4];
        
        
        x[0]=nSamples;
        x[1]=nPeriods1;
        x[2]=nSamples;
        x[3]=nPeriods2;
        
        return x;
    }
    private int mcm(int a,int b){
        //compute the minimum common multiplier
        int[] primes= new int[17];
        primes[0]=2;primes[1]=3;primes[2]=5;primes[3]=7;primes[4]=11;
        primes[5]=13;primes[6]=17;primes[7]=19;primes[8]=23;primes[9]=29;
        primes[10]=31;primes[11]=37;primes[12]=43;primes[13]=47;primes[14]=53;
        primes[15]=59;
        int mult=1, mx;
        int i=0;
        if (a<b)
            mx=a/2;
        else
            mx=b/2;
        
        while (primes[i]<mx & i<16){
            int di=primes[i];
            while( (a%di)==0 & (b%di)==0){
                a=a/di;
                b=b/di;
                mult=mult*di;}
            i++;
        }
        //System.out.println("mcm:"+a*b*mult);
        return a*b*mult;
    }
    
    
    private double computeRealFrequency(int[] x){
        //compute the actual real frequency that corresponds
        //to samples and periods
        return 44100.0*x[1]/x[0];
        //return Math.round((44100.0*x[1])/(x[0]));
    }
    
    private short[] computeSinSamples(int[] setUp,double amplitude,double phase){
        short[] samples=new short[setUp[0]];
        double xScaler=Math.PI*2*setUp[1]/(setUp[0]);
        double phaseRad=phase*Math.PI/180;
        for (int i=0;i<setUp[0];i++){
            samples[i]=(short)Math.round(Math.sin(i*xScaler-phaseRad)*amplitude);
        }
        return samples;
    }
    private short[] computeTriangleSamples(int[] setUp,double amplitude,double phase){
        short[] samples=new short[setUp[0]];
        double pi=Math.PI;
        double twoPi=pi*2;
        double xScaler=twoPi*setUp[1]/(setUp[0]);
        double phaseRad=phase*pi/180;
        double yscaler=amplitude/(pi/2);
        
        for (int i=0;i<setUp[0];i++){
            double t=(i*xScaler-phaseRad)-twoPi*Math.floor((i*xScaler-phaseRad)/twoPi);
            if (t<=pi/2)
                samples[i]=(short)Math.round(t*yscaler);
            else if (t<=pi*3/2)
                samples[i]=(short)Math.round(amplitude-(t-pi/2)*yscaler);
            else
                samples[i]=(short)Math.round(-amplitude+(t-pi*3/2)*yscaler);
            
        }
        return samples;
    }
    
    private short[] computeSquareSamples(int[] setUp,double amplitude,double phase){
        short[] samples=new short[setUp[0]];
        short sAmpl=(short)Math.round(amplitude);
        
        if (setUp[0]%setUp[1]==0){
            //we can use the jitter free approach
            //if the number of samples is odd the square signal has even harmonics
            int sp=setUp[0]/setUp[1];
            int phaseSample=(int)(phase*sp/360);
            int up=sp/2;
            //System.out.println("SP:"+sp+ "TOT: "+setUp[0]+" " +setUp[1]);
            for (int i=0;i<setUp[0];i++){
                if (((i-phaseSample)%sp)<up)
                    samples[i]=sAmpl;
                else
                    samples[i]=(short)-sAmpl;
            }
        }
        else{
            //this generates a pseudo square signal
            double xScaler=Math.PI*2*setUp[1]/(setUp[0]);
            double phaseRad=phase*Math.PI/180;
            double dv=Math.sin(1*xScaler);
            double sVal;
            //System.out.println("Jitter:"+setUp[0]+"  "+setUp[1]);
            
            for (int i=0;i<setUp[0];i++){
                double tmp=Math.sin(i*xScaler-phaseRad);
                if (tmp>0)
                    sVal=sAmpl*Math.pow(tmp,0.05);
                else
                    sVal=-sAmpl*Math.pow(-tmp,0.05);
                
                samples[i]=(short)Math.round(sVal);
            }
        }
        
        return samples;
    }
    
    class WavFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept( File f ) {
            if ( f.isDirectory() )
                return true;
            String ext=getExtension( f );
            if ( ext!=null ) {
                if ( ext.equals( "wav" ) )
                    return true;
                else
                    return false;
            }
            else
                return false;
        }
        public String getDescription() {
            return ( "Wav files" );
        }
        private String getExtension( File f ) {
            String ext = null;
            String s = f.getName();
            int i = s.lastIndexOf('.');
            if (i > 0 &&  i < s.length() - 1) {
                ext = s.substring(i+1).toLowerCase();
            }
            return ext;
        }
        
    }
    
    
    
    
}
